@@ -12,8 +12,10 @@ extern struct hardware hw_asusdh;
 extern struct hardware hw_atilibusb;
 extern struct hardware hw_audio;
 extern struct hardware hw_audio_alsa;
+extern struct hardware hw_awlibusb;
 extern struct hardware hw_bte;
 extern struct hardware hw_bw6130;
+extern struct hardware hw_commandir;
 extern struct hardware hw_creative;
 extern struct hardware hw_creative_infracd;
 extern struct hardware hw_default;
@@ -21,6 +23,7 @@ extern struct hardware hw_devinput;
 extern struct hardware hw_dsp;
 extern struct hardware hw_dvico;
 extern struct hardware hw_ea65;
+extern struct hardware hw_ftdi;
 extern struct hardware hw_i2cuser;
 extern struct hardware hw_iguanaIR;
 extern struct hardware hw_irlink;
@@ -90,10 +93,16 @@ struct hardware *hw_list[] =
 #endif
 #ifdef HAVE_LIBALSA
 	&hw_audio_alsa,
+#endif
+#ifdef HAVE_LIBUSB
+	&hw_awlibusb,
 #endif
 	&hw_bte,
 #ifdef HAVE_LINUX_HIDDEV_H
 	&hw_bw6130,
+#endif
+#ifdef HAVE_LIBUSB
+	&hw_commandir,
 #endif
 	&hw_creative,
 #ifdef HAVE_SCSI
@@ -110,7 +119,12 @@ struct hardware *hw_list[] =
 	&hw_dvico,
 #endif
 	&hw_ea65,
+#ifdef HAVE_FTDI
+	&hw_ftdi,
+#endif
+#ifdef HAVE_LINUX_I2C_DEV_H
 	&hw_i2cuser,
+#endif
 #ifdef HAVE_IGUANAIR
 	&hw_iguanaIR,
 #endif
