@@ -1,4 +1,4 @@
-/*      $Id: hw_default.c,v 5.37 2008/09/03 20:22:12 lirc Exp $      */
+/*      $Id: hw_default.c,v 5.39 2009/04/13 13:06:15 lirc Exp $      */
 
 /****************************************************************************
  ** hw_default.c ************************************************************
@@ -245,8 +245,14 @@ int default_init()
 	{
 		logprintf(LOG_ERR,"could not get hardware features");
 		logprintf(LOG_ERR,"this device driver does not "
-			  "support the new LIRC interface");
-		if(major(s.st_rdev) != LIRC_MAJOR)
+			  "support the LIRC ioctl interface");
+		if(major(s.st_rdev) == 13)
+		{
+			logprintf(LOG_ERR, "did you mean to use the devinput "
+				  "driver instead of the %s driver?",
+				  hw.name);
+		}
+		else if(major(s.st_rdev) != LIRC_MAJOR)
 		{
 			logprintf(LOG_ERR, "major number of %s is %lu",
 				  hw.device,
@@ -535,7 +541,8 @@ char *default_rec(struct ir_remote *remotes)
 			{
 				logprintf(LOG_ERR,"reading in mode "
 					  "LIRC_MODE_STRING failed");
-				return(NULL);
+				default_deinit();
+				return NULL;
 			}
 			if(n>=PACKET_SIZE-1)
 			{
@@ -551,7 +558,11 @@ char *default_rec(struct ir_remote *remotes)
 	}
 	else
 	{
-		if(!clear_rec_buffer()) return(NULL);
+		if(!clear_rec_buffer())
+		{
+			default_deinit();
+		       	return NULL;
+		}
 		return(decode_all(remotes));
 	}
 }
