@@ -22,7 +22,6 @@
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 0)
 #error "This driver needs kernel version 2.4.0 or higher"
 #endif
-
 #include <linux/autoconf.h>
 #include <linux/kernel.h>
 #include <linux/module.h>
@@ -58,18 +57,20 @@ static unsigned char do_get_bits(void);
 #define DATA_PCI_OFF 0x7FFC00
 #define WAIT_CYCLE   20
 
+#define DRIVER_NAME "lirc_bt829"
+
 static int debug;
 #define dprintk(fmt, args...)					\
 	do {							\
 		if (debug)					\
-			printk(KERN_DEBUG fmt, ## args);	\
+			printk(KERN_DEBUG DRIVER_NAME ": " fmt, ## args); \
 	} while (0)
 
 static int atir_minor;
 static unsigned long pci_addr_phys;
 static unsigned char *pci_addr_lin;
 
-static struct lirc_plugin atir_plugin;
+static struct lirc_driver atir_driver;
 
 static struct pci_dev *do_pci_probe(void)
 {
@@ -77,25 +78,25 @@ static struct pci_dev *do_pci_probe(void)
 #ifndef KERNEL_2_5
 	/* unnecessary with recent kernels */
 	if (!pci_present())
-		printk(KERN_ERR "ATIR: no pci in this kernel\n");
+		printk(KERN_ERR DRIVER_NAME ": no pci in this kernel\n");
 #endif
 	my_dev = pci_get_device(PCI_VENDOR_ID_ATI,
 				PCI_DEVICE_ID_ATI_264VT, NULL);
 	if (my_dev) {
-		printk(KERN_ERR "ATIR: Using device: %s\n",
+		printk(KERN_ERR DRIVER_NAME ": Using device: %s\n",
 		       pci_name(my_dev));
 		pci_addr_phys = 0;
 		if (my_dev->resource[0].flags & IORESOURCE_MEM) {
 			pci_addr_phys = my_dev->resource[0].start;
-			printk(KERN_INFO "ATIR memory at 0x%08X \n",
+			printk(KERN_INFO DRIVER_NAME ": memory at 0x%08X\n",
 			       (unsigned int)pci_addr_phys);
 		}
 		if (pci_addr_phys == 0) {
-			printk(KERN_ERR "ATIR no memory resource ?\n");
+			printk(KERN_ERR DRIVER_NAME ": no memory resource ?\n");
 			return NULL;
 		}
 	} else {
-		printk(KERN_ERR "ATIR: pci_prob failed\n");
+		printk(KERN_ERR DRIVER_NAME ": pci_probe failed\n");
 		return NULL;
 	}
 	return my_dev;
@@ -108,8 +109,8 @@ static int atir_add_to_buf(void *data, struct lirc_buffer *buf)
 	status = poll_main();
 	key = (status >> 8) & 0xFF;
 	if (status & 0xFF) {
-		dprintk("ATIR reading key %02X\n", key);
-		lirc_buffer_write_1(buf, &key);
+		dprintk("reading key %02X\n", key);
+		lirc_buffer_write(buf, &key);
 		return 0;
 	}
 	return -ENODATA;
@@ -118,14 +119,14 @@ static int atir_add_to_buf(void *data, struct lirc_buffer *buf)
 static int atir_set_use_inc(void *data)
 {
 	MOD_INC_USE_COUNT;
-	dprintk("ATIR driver is opened\n");
+	dprintk("driver is opened\n");
 	return 0;
 }
 
 static void atir_set_use_dec(void *data)
 {
 	MOD_DEC_USE_COUNT;
-	dprintk("ATIR driver is closed\n");
+	dprintk("driver is closed\n");
 }
 
 int init_module(void)
@@ -139,21 +140,25 @@ int init_module(void)
 	if (!atir_init_start())
 		return 1;
 
-	strcpy(atir_plugin.name, "ATIR");
-	atir_plugin.minor       = -1;
-	atir_plugin.code_length = 8;
-	atir_plugin.sample_rate = 10;
-	atir_plugin.data        = 0;
-	atir_plugin.add_to_buf  = atir_add_to_buf;
-	atir_plugin.set_use_inc = atir_set_use_inc;
-	atir_plugin.set_use_dec = atir_set_use_dec;
+	strcpy(atir_driver.name, "ATIR");
+	atir_driver.minor       = -1;
+	atir_driver.code_length = 8;
+	atir_driver.sample_rate = 10;
+	atir_driver.data        = 0;
+	atir_driver.add_to_buf  = atir_add_to_buf;
+	atir_driver.set_use_inc = atir_set_use_inc;
+	atir_driver.set_use_dec = atir_set_use_dec;
 #ifdef LIRC_HAVE_SYSFS
-	atir_plugin.dev         = &pdev->dev;
+	atir_driver.dev         = &pdev->dev;
 #endif
-	atir_plugin.owner       = THIS_MODULE;
+	atir_driver.owner       = THIS_MODULE;
 
-	atir_minor = lirc_register_plugin(&atir_plugin);
-	dprintk("ATIR driver is registered on minor %d\n", atir_minor);
+	atir_minor = lirc_register_driver(&atir_driver);
+	if (atir_minor < 0) {
+		printk(KERN_ERR DRIVER_NAME ": failed to register driver!\n");
+		return atir_minor;
+	}
+	dprintk("driver is registered on minor %d\n", atir_minor);
 
 	return 0;
 }
@@ -161,7 +166,7 @@ int init_module(void)
 
 void cleanup_module(void)
 {
-	lirc_unregister_plugin(atir_minor);
+	lirc_unregister_driver(atir_minor);
 }
 
 
@@ -169,7 +174,7 @@ static int atir_init_start(void)
 {
 	pci_addr_lin = ioremap(pci_addr_phys + DATA_PCI_OFF, 0x400);
 	if (pci_addr_lin == 0) {
-		printk(KERN_INFO "atir: pci mem must be mapped\n");
+		printk(KERN_INFO DRIVER_NAME ": pci mem must be mapped\n");
 		return 0;
 	}
 	return 1;
@@ -333,11 +338,11 @@ static void do_set_bits(unsigned char new_bits)
 	write_index(0x34, reg_val);
 
 	reg_val = read_index(0x31);
-	if (new_bits & 1) {
+	if (new_bits & 1)
 		reg_val |= 0x1000000;
-	} else {
+	else
 		reg_val &= 0xFEFFFFFF;
-	}
+
 	reg_val |= 0x8000000;
 	write_index(0x31, reg_val);
 }
@@ -354,17 +359,17 @@ static unsigned char do_get_bits(void)
 
 	reg_val = read_index(0x34);
 	bits = 0;
-	if (reg_val & 8) {
+	if (reg_val & 8)
 		bits |= 2;
-	} else {
+	else
 		bits &= 0xFD;
-	}
+
 	reg_val = read_index(0x31);
-	if (reg_val & 0x1000000) {
+	if (reg_val & 0x1000000)
 		bits |= 1;
-	} else {
+	else
 		bits &= 0xFE;
-	}
+
 	return bits;
 }
 
@@ -389,15 +394,6 @@ MODULE_AUTHOR("Froenchenko Leonid");
 MODULE_DESCRIPTION("IR remote driver for bt829 based TV cards");
 MODULE_LICENSE("GPL");
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Debug enabled or not");
-
 EXPORT_NO_SYMBOLS;
-
-/*
- * Overrides for Emacs so that we follow Linus's tabbing style.
- * ---------------------------------------------------------------------------
- * Local variables:
- * c-basic-offset: 8
- * End:
- */
