@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
- * $Id: lirc_dev.c,v 1.61 2008/09/27 08:16:15 lirc Exp $
+ * $Id: lirc_dev.c,v 1.96 2009/08/31 16:57:55 lirc Exp $
  *
  */
 
@@ -49,11 +49,6 @@
 #include <linux/uaccess.h>
 #include <linux/errno.h>
 #endif
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 27)
-#include <asm/semaphore.h>
-#else
-#include <linux/semaphore.h>
-#endif
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 0)
 #include <linux/wrapper.h>
 #endif
@@ -62,6 +57,9 @@
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 23)
 #include <linux/kthread.h>
 #endif
+#ifdef CONFIG_COMPAT
+#include <linux/compat.h>
+#endif
 
 #include "drivers/kcompat.h"
 
@@ -81,17 +79,17 @@ static int debug;
 	} while (0)
 
 #define IRCTL_DEV_NAME    "BaseRemoteCtl"
-#define SUCCESS           0
 #define NOPLUG            -1
 #define LOGHEAD           "lirc_dev (%s[%d]): "
 
 struct irctl {
-	struct lirc_plugin p;
+	struct lirc_driver d;
 	int attached;
 	int open;
 
-	struct semaphore buffer_sem;
+	struct mutex buffer_lock;
 	struct lirc_buffer *buf;
+	unsigned int chunk_size;
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	int tpid;
@@ -108,9 +106,9 @@ struct irctl {
 #endif
 };
 
-DECLARE_MUTEX(plugin_lock);
+static DEFINE_MUTEX(lirc_dev_lock);
 
-static struct irctl irctls[MAX_IRCTL_DEVICES];
+static struct irctl *irctls[MAX_IRCTL_DEVICES];
 static struct file_operations fops;
 
 /* Only used for sysfs but defined to void otherwise */
@@ -119,65 +117,42 @@ static lirc_class_t *lirc_class;
 /*  helper function
  *  initializes the irctl structure
  */
-static inline void init_irctl(struct irctl *ir)
+static void init_irctl(struct irctl *ir)
 {
-	memset(&ir->p, 0, sizeof(struct lirc_plugin));
-	sema_init(&ir->buffer_sem, 1);
-	ir->p.minor = NOPLUG;
-
+	mutex_init(&ir->buffer_lock);
+	ir->d.minor = NOPLUG;
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	ir->tpid = -1;
-	ir->t_notify = NULL;
-	ir->t_notify2 = NULL;
-	ir->shutdown = 0;
-#else
-	ir->task = NULL;
 #endif
-	ir->jiffies_to_wait = 0;
-
-	ir->open = 0;
-	ir->attached = 0;
 }
 
 static void cleanup(struct irctl *ir)
 {
-	dprintk(LOGHEAD "cleaning up\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "cleaning up\n", ir->d.name, ir->d.minor);
 
-	if (ir->buf != ir->p.rbuf) {
+	if (ir->buf != ir->d.rbuf) {
 		lirc_buffer_free(ir->buf);
 		kfree(ir->buf);
 	}
 	ir->buf = NULL;
-
-	init_irctl(ir);
 }
 
 /*  helper function
- *  reads key codes from plugin and puts them into buffer
- *  buffer free space is checked and locking performed
+ *  reads key codes from driver and puts them into buffer
  *  returns 0 on success
  */
-static inline int add_to_buf(struct irctl *ir)
+static int add_to_buf(struct irctl *ir)
 {
-	if (lirc_buffer_full(ir->buf)) {
-		dprintk(LOGHEAD "buffer overflow\n",
-			ir->p.name, ir->p.minor);
-		return -EOVERFLOW;
-	}
-
-	if (ir->p.add_to_buf) {
+	if (ir->d.add_to_buf) {
 		int res = -ENODATA;
 		int got_data = 0;
 
-		/* service the device as long as it is returning
+		/*
+		 * service the device as long as it is returning
 		 * data and we have space
 		 */
-		while (!lirc_buffer_full(ir->buf)) {
-			res = ir->p.add_to_buf(ir->p.data, ir->buf);
-			if (res == SUCCESS)
-				got_data++;
-			else
-				break;
+		while ((res = ir->d.add_to_buf(ir->d.data, ir->buf)) == 0) {
+			got_data++;
 		}
 
 		if (res == -ENODEV)
@@ -187,38 +162,40 @@ static inline int add_to_buf(struct irctl *ir)
 			kthread_stop(ir->task);
 #endif
 
-		return (got_data ? SUCCESS : res);
+		return got_data ? 0 : res;
 	}
 
-	return SUCCESS;
+	return 0;
 }
 
-/* main function of the polling thread
- */
+/* main function of the polling thread */
 static int lirc_thread(void *irctl)
 {
 	struct irctl *ir = irctl;
 
-	/* This thread doesn't need any user-level access,
-	 * so get rid of all our resources
-	 */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
+	/*
+	 * This thread doesn't need any user-level access, so get rid
+	 * of all our resources
+	 */
 	daemonize("lirc_dev");
 
 	if (ir->t_notify != NULL)
 		complete(ir->t_notify);
+	
 #endif
-
-	dprintk(LOGHEAD "poll thread started\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "poll thread started\n", ir->d.name, ir->d.minor);
 
 	do {
 		if (ir->open) {
 			if (ir->jiffies_to_wait) {
 				set_current_state(TASK_INTERRUPTIBLE);
 				schedule_timeout(ir->jiffies_to_wait);
+#ifndef LIRC_REMOVE_DURING_EXPORT
 			} else {
 				interruptible_sleep_on(
-					ir->p.get_queue(ir->p.data));
+					ir->d.get_queue(ir->d.data));
+#endif
 			}
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 			if (ir->shutdown)
@@ -229,9 +206,8 @@ static int lirc_thread(void *irctl)
 			if (!add_to_buf(ir))
 				wake_up_interruptible(&ir->buf->wait_poll);
 		} else {
-			/* if device not opened so we can sleep half a second */
 			set_current_state(TASK_INTERRUPTIBLE);
-			schedule_timeout(HZ/2);
+			schedule();
 		}
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	} while (!ir->shutdown);
@@ -246,161 +222,177 @@ static int lirc_thread(void *irctl)
 	} while (!kthread_should_stop());
 #endif
 
-	dprintk(LOGHEAD "poll thread ended\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "poll thread ended\n", ir->d.name, ir->d.minor);
 
 	return 0;
 }
 
-int lirc_register_plugin(struct lirc_plugin *p)
+int lirc_register_driver(struct lirc_driver *d)
 {
 	struct irctl *ir;
 	int minor;
 	int bytes_in_key;
+	unsigned int buffer_size;
 	int err;
 #ifdef LIRC_HAVE_DEVFS_24
 	char name[16];
 #endif
 	DECLARE_COMPLETION(tn);
 
-	if (!p) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
-		       "plugin pointer must be not NULL!\n");
+	if (!d) {
+		printk(KERN_ERR "lirc_dev: lirc_register_driver: "
+		       "driver pointer must be not NULL!\n");
 		err = -EBADRQC;
 		goto out;
 	}
 
-	if (MAX_IRCTL_DEVICES <= p->minor) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+	if (MAX_IRCTL_DEVICES <= d->minor) {
+		printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 		       "\"minor\" must be between 0 and %d (%d)!\n",
-		       MAX_IRCTL_DEVICES-1, p->minor);
+		       MAX_IRCTL_DEVICES-1, d->minor);
 		err = -EBADRQC;
 		goto out;
 	}
 
-	if (1 > p->code_length || (BUFLEN * 8) < p->code_length) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+	if (1 > d->code_length || (BUFLEN * 8) < d->code_length) {
+		printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 		       "code length in bits for minor (%d) "
 		       "must be less than %d!\n",
-		       p->minor, BUFLEN * 8);
+		       d->minor, BUFLEN * 8);
 		err = -EBADRQC;
 		goto out;
 	}
 
-	printk(KERN_INFO "lirc_dev: lirc_register_plugin: sample_rate: %d\n",
-		p->sample_rate);
-	if (p->sample_rate) {
-		if (2 > p->sample_rate || HZ < p->sample_rate) {
-			printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+	printk(KERN_INFO "lirc_dev: lirc_register_driver: sample_rate: %d\n",
+		d->sample_rate);
+	if (d->sample_rate) {
+		if (2 > d->sample_rate || HZ < d->sample_rate) {
+			printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 			       "sample_rate must be between 2 and %d!\n", HZ);
 			err = -EBADRQC;
 			goto out;
 		}
-		if (!p->add_to_buf) {
-			printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+		if (!d->add_to_buf) {
+			printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 			       "add_to_buf cannot be NULL when "
 			       "sample_rate is set\n");
 			err = -EBADRQC;
 			goto out;
 		}
-	} else if (!(p->fops && p->fops->read)
-		   && !p->get_queue && !p->rbuf) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+#ifndef LIRC_REMOVE_DURING_EXPORT
+	} else if (!(d->fops && d->fops->read) && !d->get_queue && !d->rbuf) {
+#else
+	} else if (!(d->fops && d->fops->read) && !d->rbuf) {
+#endif
+		printk(KERN_ERR "lirc_dev: lirc_register_driver: "
+#ifndef LIRC_REMOVE_DURING_EXPORT
 		       "fops->read, get_queue and rbuf "
+#else
+		       "fops->read and rbuf "
+#endif
 		       "cannot all be NULL!\n");
 		err = -EBADRQC;
 		goto out;
-	} else if (!p->get_queue && !p->rbuf) {
-		if (!(p->fops && p->fops->read && p->fops->poll)
-		    || (!p->fops->ioctl && !p->ioctl)) {
-			printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+#ifndef LIRC_REMOVE_DURING_EXPORT
+	} else if (!d->get_queue && !d->rbuf) {
+#else
+	} else if (!d->rbuf) {
+#endif
+		if (!(d->fops && d->fops->read && d->fops->poll &&
+		      d->fops->ioctl)) {
+			printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 			       "neither read, poll nor ioctl can be NULL!\n");
 			err = -EBADRQC;
 			goto out;
 		}
 	}
 
-	if (p->owner == NULL) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
-				    "no module owner registered\n");
-		err = -EBADRQC;
-		goto out;
-	}
+	mutex_lock(&lirc_dev_lock);
 
-	down(&plugin_lock);
+	minor = d->minor;
 
-	minor = p->minor;
-
-	if (0 > minor) {
-		/* find first free slot for plugin */
+	if (minor < 0) {
+		/* find first free slot for driver */
 		for (minor = 0; minor < MAX_IRCTL_DEVICES; minor++)
-			if (irctls[minor].p.minor == NOPLUG)
+			if (!irctls[minor])
 				break;
 		if (MAX_IRCTL_DEVICES == minor) {
-			printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
-			       "no free slots for plugins!\n");
+			printk(KERN_ERR "lirc_dev: lirc_register_driver: "
+			       "no free slots for drivers!\n");
 			err = -ENOMEM;
 			goto out_lock;
 		}
-	} else if (irctls[minor].p.minor != NOPLUG) {
-		printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+	} else if (irctls[minor]) {
+		printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 		       "minor (%d) just registered!\n", minor);
 		err = -EBUSY;
 		goto out_lock;
 	}
 
-	ir = &irctls[minor];
+	ir = kzalloc(sizeof(struct irctl), GFP_KERNEL);
+	if (!ir) {
+		err = -ENOMEM;
+		goto out_lock;
+	}
+	init_irctl(ir);
+	irctls[minor] = ir;
 
-	if (p->sample_rate) {
-		ir->jiffies_to_wait = HZ / p->sample_rate;
+	if (d->sample_rate) {
+		ir->jiffies_to_wait = HZ / d->sample_rate;
 	} else {
 		/* it means - wait for external event in task queue */
 		ir->jiffies_to_wait = 0;
 	}
 
 	/* some safety check 8-) */
-	p->name[sizeof(p->name)-1] = '\0';
+	d->name[sizeof(d->name)-1] = '\0';
 
-	bytes_in_key = p->code_length/8 + (p->code_length%8 ? 1 : 0);
+	bytes_in_key = d->code_length/8 + (d->code_length%8 ? 1 : 0);
+	buffer_size = d->buffer_size ? d->buffer_size : BUFLEN / bytes_in_key;
 
-	if (p->rbuf) {
-		ir->buf = p->rbuf;
+	if (d->rbuf) {
+		ir->buf = d->rbuf;
 	} else {
 		ir->buf = kmalloc(sizeof(struct lirc_buffer), GFP_KERNEL);
 		if (!ir->buf) {
 			err = -ENOMEM;
 			goto out_lock;
 		}
-		if (lirc_buffer_init(ir->buf, bytes_in_key,
-				     BUFLEN/bytes_in_key) != 0) {
+		err = lirc_buffer_init(ir->buf, bytes_in_key, buffer_size);
+		if (err) {
 			kfree(ir->buf);
-			err = -ENOMEM;
 			goto out_lock;
 		}
 	}
+	ir->chunk_size = ir->buf->chunk_size;
 
-	if (p->features == 0)
-		p->features = (p->code_length > 8) ?
+	if (d->features == 0)
+		d->features = (d->code_length > 8) ?
 			LIRC_CAN_REC_LIRCCODE : LIRC_CAN_REC_CODE;
 
-	ir->p = *p;
-	ir->p.minor = minor;
+	ir->d = *d;
+	ir->d.minor = minor;
 
 #if defined(LIRC_HAVE_DEVFS_24)
-	sprintf(name, DEV_LIRC "/%d", ir->p.minor);
+	sprintf(name, DEV_LIRC "/%d", ir->d.minor);
 	ir->devfs_handle = devfs_register(NULL, name, DEVFS_FL_DEFAULT,
-					  IRCTL_DEV_MAJOR, ir->p.minor,
+					  IRCTL_DEV_MAJOR, ir->d.minor,
 					  S_IFCHR | S_IRUSR | S_IWUSR,
 					  &fops, NULL);
 #elif defined(LIRC_HAVE_DEVFS_26)
-	devfs_mk_cdev(MKDEV(IRCTL_DEV_MAJOR, ir->p.minor),
+	devfs_mk_cdev(MKDEV(IRCTL_DEV_MAJOR, ir->d.minor),
 			S_IFCHR|S_IRUSR|S_IWUSR,
-			DEV_LIRC "/%u", ir->p.minor);
+			DEV_LIRC "/%u", ir->d.minor);
 #endif
-	(void) lirc_device_create(lirc_class, ir->p.dev,
-				  MKDEV(IRCTL_DEV_MAJOR, ir->p.minor), NULL,
-				  "lirc%u", ir->p.minor);
+	(void) lirc_device_create(lirc_class, ir->d.dev,
+				  MKDEV(IRCTL_DEV_MAJOR, ir->d.minor), NULL,
+				  "lirc%u", ir->d.minor);
 
-	if (p->sample_rate || p->get_queue) {
+#ifndef LIRC_REMOVE_DURING_EXPORT
+	if (d->sample_rate || d->get_queue) {
+#else
+	if (d->sample_rate) {
+#endif
 		/* try to fire up polling thread */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 		ir->t_notify = &tn;
@@ -410,9 +402,9 @@ int lirc_register_plugin(struct lirc_plugin *p)
 		ir->task = kthread_run(lirc_thread, (void *)ir, "lirc_dev");
 		if (IS_ERR(ir->task)) {
 #endif
-			printk(KERN_ERR "lirc_dev: lirc_register_plugin: "
+			printk(KERN_ERR "lirc_dev: lirc_register_driver: "
 			       "cannot run poll thread for minor = %d\n",
-			       p->minor);
+			       d->minor);
 			err = -ECHILD;
 			goto out_sysfs;
 		}
@@ -422,7 +414,7 @@ int lirc_register_plugin(struct lirc_plugin *p)
 #endif
 	}
 	ir->attached = 1;
-	up(&plugin_lock);
+	mutex_unlock(&lirc_dev_lock);
 
 /*
  * Recent kernels should handle this autmatically by increasing/decreasing
@@ -431,48 +423,50 @@ int lirc_register_plugin(struct lirc_plugin *p)
 #ifndef KERNEL_2_5
 	MOD_INC_USE_COUNT;
 #endif
-	dprintk("lirc_dev: plugin %s registered at minor number = %d\n",
-		ir->p.name, ir->p.minor);
-	p->minor = minor;
+	dprintk("lirc_dev: driver %s registered at minor number = %d\n",
+		ir->d.name, ir->d.minor);
+	d->minor = minor;
 	return minor;
 
 out_sysfs:
 	lirc_device_destroy(lirc_class,
-			    MKDEV(IRCTL_DEV_MAJOR, ir->p.minor));
+			    MKDEV(IRCTL_DEV_MAJOR, ir->d.minor));
 #ifdef LIRC_HAVE_DEVFS_24
 	devfs_unregister(ir->devfs_handle);
 #endif
 #ifdef LIRC_HAVE_DEVFS_26
-	devfs_remove(DEV_LIRC "/%i", ir->p.minor);
+	devfs_remove(DEV_LIRC "/%i", ir->d.minor);
 #endif
 out_lock:
-	up(&plugin_lock);
+	mutex_unlock(&lirc_dev_lock);
 out:
 	return err;
 }
-EXPORT_SYMBOL(lirc_register_plugin);
+EXPORT_SYMBOL(lirc_register_driver);
 
-int lirc_unregister_plugin(int minor)
+int lirc_unregister_driver(int minor)
 {
 	struct irctl *ir;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	DECLARE_COMPLETION(tn);
 	DECLARE_COMPLETION(tn2);
+#endif
 
 	if (minor < 0 || minor >= MAX_IRCTL_DEVICES) {
-		printk(KERN_ERR "lirc_dev: lirc_unregister_plugin: "
+		printk(KERN_ERR "lirc_dev: lirc_unregister_driver: "
 		       "\"minor\" must be between 0 and %d!\n",
 		       MAX_IRCTL_DEVICES-1);
 		return -EBADRQC;
 	}
 
-	ir = &irctls[minor];
+	ir = irctls[minor];
 
-	down(&plugin_lock);
+	mutex_lock(&lirc_dev_lock);
 
-	if (ir->p.minor != minor) {
-		printk(KERN_ERR "lirc_dev: lirc_unregister_plugin: "
+	if (ir->d.minor != minor) {
+		printk(KERN_ERR "lirc_dev: lirc_unregister_driver: "
 		       "minor (%d) device not registered!", minor);
-		up(&plugin_lock);
+		mutex_unlock(&lirc_dev_lock);
 		return -ENOENT;
 	}
 
@@ -491,7 +485,7 @@ int lirc_unregister_plugin(int minor)
 		}
 #else
 		/* 2.2.x does not export wake_up_process() */
-		wake_up_interruptible(ir->p.get_queue(ir->p.data));
+		wake_up_interruptible(ir->d.get_queue(ir->d.data));
 #endif
 		complete(&tn2);
 		wait_for_completion(&tn);
@@ -499,38 +493,38 @@ int lirc_unregister_plugin(int minor)
 		ir->t_notify2 = NULL;
 	}
 #else /* kernel >= 2.6.23 */
-	if (ir->task) {
-		wake_up_process(ir->task);
+	if (ir->task)
 		kthread_stop(ir->task);
-	}
 #endif
 
-	dprintk("lirc_dev: plugin %s unregistered from minor number = %d\n",
-		ir->p.name, ir->p.minor);
+	dprintk("lirc_dev: driver %s unregistered from minor number = %d\n",
+		ir->d.name, ir->d.minor);
 
 	ir->attached = 0;
 	if (ir->open) {
-		dprintk(LOGHEAD "releasing opened plugin\n",
-			ir->p.name, ir->p.minor);
+		dprintk(LOGHEAD "releasing opened driver\n",
+			ir->d.name, ir->d.minor);
 		wake_up_interruptible(&ir->buf->wait_poll);
-		down(&ir->buffer_sem);
-		ir->p.set_use_dec(ir->p.data);
-		module_put(ir->p.owner);
-		up(&ir->buffer_sem);
+		mutex_lock(&ir->buffer_lock);
+		ir->d.set_use_dec(ir->d.data);
+		module_put(ir->d.owner);
+		mutex_unlock(&ir->buffer_lock);
+	} else {
+		cleanup(ir);
+		irctls[minor] = NULL;
+		kfree(ir);
 	}
 
 #ifdef LIRC_HAVE_DEVFS_24
 	devfs_unregister(ir->devfs_handle);
 #endif
 #ifdef LIRC_HAVE_DEVFS_26
-	devfs_remove(DEV_LIRC "/%u", ir->p.minor);
+	devfs_remove(DEV_LIRC "/%u", ir->d.minor);
 #endif
 	lirc_device_destroy(lirc_class,
-			    MKDEV(IRCTL_DEV_MAJOR, ir->p.minor));
+			    MKDEV(IRCTL_DEV_MAJOR, ir->d.minor));
 
-	if (!ir->open)
-		cleanup(ir);
-	up(&plugin_lock);
+	mutex_unlock(&lirc_dev_lock);
 
 /*
  * Recent kernels should handle this autmatically by increasing/decreasing
@@ -540,137 +534,128 @@ int lirc_unregister_plugin(int minor)
 	MOD_DEC_USE_COUNT;
 #endif
 
-	return SUCCESS;
+	return 0;
 }
-EXPORT_SYMBOL(lirc_unregister_plugin);
+EXPORT_SYMBOL(lirc_unregister_driver);
 
-/*
- *
- */
 static int irctl_open(struct inode *inode, struct file *file)
 {
 	struct irctl *ir;
 	int retval;
 
-	if (MINOR(inode->i_rdev) >= MAX_IRCTL_DEVICES) {
+	if (iminor(inode) >= MAX_IRCTL_DEVICES || !irctls[iminor(inode)]) {
 		dprintk("lirc_dev [%d]: open result = -ENODEV\n",
-			MINOR(inode->i_rdev));
+			iminor(inode));
 		return -ENODEV;
 	}
 
-	ir = &irctls[MINOR(inode->i_rdev)];
+	ir = irctls[iminor(inode)];
 
-	dprintk(LOGHEAD "open called\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "open called\n", ir->d.name, ir->d.minor);
 
-	/* if the plugin has an open function use it instead */
-	if (ir->p.fops && ir->p.fops->open)
-		return ir->p.fops->open(inode, file);
+	/* if the driver has an open function use it instead */
+	if (ir->d.fops && ir->d.fops->open)
+		return ir->d.fops->open(inode, file);
 
-	if (down_interruptible(&plugin_lock))
+	if (mutex_lock_interruptible(&lirc_dev_lock))
 		return -ERESTARTSYS;
 
-	if (ir->p.minor == NOPLUG) {
-		up(&plugin_lock);
-		dprintk(LOGHEAD "open result = -ENODEV\n",
-			ir->p.name, ir->p.minor);
-		return -ENODEV;
+	if (ir->d.minor == NOPLUG) {
+		retval = -ENODEV;
+		goto error;
 	}
 
 	if (ir->open) {
-		up(&plugin_lock);
-		dprintk(LOGHEAD "open result = -EBUSY\n",
-			ir->p.name, ir->p.minor);
-		return -EBUSY;
+		retval = -EBUSY;
+		goto error;
 	}
 
-	/* there is no need for locking here because ir->open is 0
-	 * and lirc_thread isn't using buffer
-	 * plugins which use irq's should allocate them on set_use_inc,
-	 * so there should be no problem with those either.
-	 */
-	ir->buf->head = ir->buf->tail;
-	ir->buf->fill = 0;
-
-	if (ir->p.owner != NULL && try_module_get(ir->p.owner)) {
+	if (try_module_get(ir->d.owner)) {
 		++ir->open;
-		retval = ir->p.set_use_inc(ir->p.data);
+		retval = ir->d.set_use_inc(ir->d.data);
 
-		if (retval != SUCCESS) {
-			module_put(ir->p.owner);
+		if (retval) {
+			module_put(ir->d.owner);
 			--ir->open;
+		} else {
+			lirc_buffer_clear(ir->buf);
 		}
-	} else {
-		if (ir->p.owner == NULL)
-			dprintk(LOGHEAD "no module owner!!!\n",
-				ir->p.name, ir->p.minor);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
+		if (ir->tpid >= 0) {
+			struct task_struct *p;
 
-		retval = -ENODEV;
+			p = find_task_by_pid(ir->tpid);
+			wake_up_process(p);
+		}
+#else
+		if (ir->task)
+			wake_up_process(ir->task);
+#endif
 	}
+ error:
+	if (ir)
+		dprintk(LOGHEAD "open result = %d\n", ir->d.name, ir->d.minor,
+			retval);
 
-	dprintk(LOGHEAD "open result = %d\n", ir->p.name, ir->p.minor, retval);
-	up(&plugin_lock);
+	mutex_unlock(&lirc_dev_lock);
 
 	return retval;
 }
 
-/*
- *
- */
 static int irctl_close(struct inode *inode, struct file *file)
 {
-	struct irctl *ir = &irctls[MINOR(inode->i_rdev)];
+	struct irctl *ir = irctls[iminor(inode)];
 
-	dprintk(LOGHEAD "close called\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "close called\n", ir->d.name, ir->d.minor);
 
-	/* if the plugin has a close function use it instead */
-	if (ir->p.fops && ir->p.fops->release)
-		return ir->p.fops->release(inode, file);
+	/* if the driver has a close function use it instead */
+	if (ir->d.fops && ir->d.fops->release)
+		return ir->d.fops->release(inode, file);
 
-	if (down_interruptible(&plugin_lock))
+	if (mutex_lock_interruptible(&lirc_dev_lock))
 		return -ERESTARTSYS;
 
 	--ir->open;
 	if (ir->attached) {
-		ir->p.set_use_dec(ir->p.data);
-		module_put(ir->p.owner);
+		ir->d.set_use_dec(ir->d.data);
+		module_put(ir->d.owner);
 	} else {
 		cleanup(ir);
+		irctls[ir->d.minor] = NULL;
+		kfree(ir);
 	}
 
-	up(&plugin_lock);
+	mutex_unlock(&lirc_dev_lock);
 
-	return SUCCESS;
+	return 0;
 }
 
-/*
- *
- */
 static unsigned int irctl_poll(struct file *file, poll_table *wait)
 {
-	struct irctl *ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
+	struct irctl *ir = irctls[iminor(file->f_dentry->d_inode)];
 	unsigned int ret;
 
-	dprintk(LOGHEAD "poll called\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "poll called\n", ir->d.name, ir->d.minor);
 
-	/* if the plugin has a poll function use it instead */
-	if (ir->p.fops && ir->p.fops->poll)
-		return ir->p.fops->poll(file, wait);
+	/* if the driver has a poll function use it instead */
+	if (ir->d.fops && ir->d.fops->poll)
+		return ir->d.fops->poll(file, wait);
 
-	down(&ir->buffer_sem);
+	mutex_lock(&ir->buffer_lock);
 	if (!ir->attached) {
-		up(&ir->buffer_sem);
+		mutex_unlock(&ir->buffer_lock);
 		return POLLERR;
 	}
 
 	poll_wait(file, &ir->buf->wait_poll, wait);
 
 	dprintk(LOGHEAD "poll result = %s\n",
-		ir->p.name, ir->p.minor,
+		ir->d.name, ir->d.minor,
 		lirc_buffer_empty(ir->buf) ? "0" : "POLLIN|POLLRDNORM");
 
 	ret = lirc_buffer_empty(ir->buf) ? 0 : (POLLIN|POLLRDNORM);
 
-	up(&ir->buffer_sem);
+	mutex_unlock(&ir->buffer_lock);
 	return ret;
 }
 
@@ -682,48 +667,45 @@ static int irctl_ioctl(struct inode *inode, struct file *file,
 {
 	unsigned long mode;
 	int result;
-	struct irctl *ir = &irctls[MINOR(inode->i_rdev)];
+	struct irctl *ir = irctls[iminor(inode)];
 
 	dprintk(LOGHEAD "ioctl called (0x%x)\n",
-		ir->p.name, ir->p.minor, cmd);
+		ir->d.name, ir->d.minor, cmd);
 
-	/* if the plugin has a ioctl function use it instead */
-	if (ir->p.fops && ir->p.fops->ioctl)
-		return ir->p.fops->ioctl(inode, file, cmd, arg);
-
-	if (ir->p.minor == NOPLUG || !ir->attached) {
-		dprintk(LOGHEAD "ioctl result = -ENODEV\n",
-			ir->p.name, ir->p.minor);
-		return -ENODEV;
-	}
-
-	/* Give the plugin a chance to handle the ioctl */
-	if (ir->p.ioctl) {
-		result = ir->p.ioctl(inode, file, cmd, arg);
+	/* if the driver has a ioctl function use it instead */
+	if (ir->d.fops && ir->d.fops->ioctl) {
+		result = ir->d.fops->ioctl(inode, file, cmd, arg);
 		if (result != -ENOIOCTLCMD)
 			return result;
 	}
-	/* The plugin can't handle cmd */
-	result = SUCCESS;
+
+	if (ir->d.minor == NOPLUG || !ir->attached) {
+		dprintk(LOGHEAD "ioctl result = -ENODEV\n",
+			ir->d.name, ir->d.minor);
+		return -ENODEV;
+	}
+
+	/* The driver can't handle cmd */
+	result = 0;
 
 	switch (cmd) {
 	case LIRC_GET_FEATURES:
-		result = put_user(ir->p.features, (unsigned long *)arg);
+		result = put_user(ir->d.features, (unsigned long *)arg);
 		break;
 	case LIRC_GET_REC_MODE:
-		if (!(ir->p.features&LIRC_CAN_REC_MASK))
+		if (!(ir->d.features & LIRC_CAN_REC_MASK))
 			return -ENOSYS;
 
 		result = put_user(LIRC_REC2MODE
-				  (ir->p.features&LIRC_CAN_REC_MASK),
+				  (ir->d.features & LIRC_CAN_REC_MASK),
 				  (unsigned long *)arg);
 		break;
 	case LIRC_SET_REC_MODE:
-		if (!(ir->p.features&LIRC_CAN_REC_MASK))
+		if (!(ir->d.features & LIRC_CAN_REC_MASK))
 			return -ENOSYS;
 
 		result = get_user(mode, (unsigned long *)arg);
-		if (!result && !(LIRC_MODE2REC(mode) & ir->p.features))
+		if (!result && !(LIRC_MODE2REC(mode) & ir->d.features))
 			result = -EINVAL;
 		/*
 		 * FIXME: We should actually set the mode somehow but
@@ -731,62 +713,151 @@ static int irctl_ioctl(struct inode *inode, struct file *file,
 		 */
 		break;
 	case LIRC_GET_LENGTH:
-		result = put_user((unsigned long)ir->p.code_length,
-				  (unsigned long *)arg);
+		result = put_user(ir->d.code_length, (unsigned long *) arg);
 		break;
 	default:
-		result = -ENOIOCTLCMD;
+		result = -EINVAL;
 	}
 
 	dprintk(LOGHEAD "ioctl result = %d\n",
-		ir->p.name, ir->p.minor, result);
+		ir->d.name, ir->d.minor, result);
 
 	return result;
 }
 
-/*
- *
- */
+#ifdef CONFIG_COMPAT
+#define LIRC_GET_FEATURES_COMPAT32     _IOR('i', 0x00000000, __u32)
+
+#define LIRC_GET_SEND_MODE_COMPAT32    _IOR('i', 0x00000001, __u32)
+#define LIRC_GET_REC_MODE_COMPAT32     _IOR('i', 0x00000002, __u32)
+
+#define LIRC_GET_LENGTH_COMPAT32       _IOR('i', 0x0000000f, __u32)
+
+#define LIRC_SET_SEND_MODE_COMPAT32    _IOW('i', 0x00000011, __u32)
+#define LIRC_SET_REC_MODE_COMPAT32     _IOW('i', 0x00000012, __u32)
+
+static long irctl_compat_ioctl(struct file *file,
+			       unsigned int cmd32,
+			       unsigned long arg)
+{
+	mm_segment_t old_fs;
+	int ret;
+	unsigned long val;
+	unsigned int cmd;
+
+	switch (cmd32) {
+	case LIRC_GET_FEATURES_COMPAT32:
+	case LIRC_GET_SEND_MODE_COMPAT32:
+	case LIRC_GET_REC_MODE_COMPAT32:
+	case LIRC_GET_LENGTH_COMPAT32:
+	case LIRC_SET_SEND_MODE_COMPAT32:
+	case LIRC_SET_REC_MODE_COMPAT32:
+		/*
+		 * These commands expect (unsigned long *) arg
+		 * but the 32-bit app supplied (__u32 *).
+		 * Conversion is required.
+		 */
+		if (get_user(val, (__u32 *)compat_ptr(arg)))
+			return -EFAULT;
+		lock_kernel();
+		/*
+		 * tell irctl_ioctl that it's safe to use the pointer
+		 * to val which is in kernel address space and not in
+		 * user address space
+		 */
+		old_fs = get_fs();
+		set_fs(KERNEL_DS);
+
+		cmd = _IOC(_IOC_DIR(cmd32), _IOC_TYPE(cmd32), _IOC_NR(cmd32),
+			   (_IOC_TYPECHECK(unsigned long)));
+		ret = irctl_ioctl(file->f_dentry->d_inode, file,
+				  cmd, (unsigned long)(&val));
+
+		set_fs(old_fs);
+		unlock_kernel();
+		switch (cmd) {
+		case LIRC_GET_FEATURES:
+		case LIRC_GET_SEND_MODE:
+		case LIRC_GET_REC_MODE:
+		case LIRC_GET_LENGTH:
+			if (!ret && put_user(val, (__u32 *)compat_ptr(arg)))
+				return -EFAULT;
+			break;
+		}
+		return ret;
+
+	case LIRC_GET_SEND_CARRIER:
+	case LIRC_GET_REC_CARRIER:
+	case LIRC_GET_SEND_DUTY_CYCLE:
+	case LIRC_GET_REC_DUTY_CYCLE:
+	case LIRC_GET_REC_RESOLUTION:
+	case LIRC_SET_SEND_CARRIER:
+	case LIRC_SET_REC_CARRIER:
+	case LIRC_SET_SEND_DUTY_CYCLE:
+	case LIRC_SET_REC_DUTY_CYCLE:
+	case LIRC_SET_TRANSMITTER_MASK:
+	case LIRC_SET_REC_DUTY_CYCLE_RANGE:
+	case LIRC_SET_REC_CARRIER_RANGE:
+		/*
+		 * These commands expect (unsigned int *)arg
+		 * so no problems here. Just handle the locking.
+		 */
+		lock_kernel();
+		cmd = cmd32;
+		ret = irctl_ioctl(file->f_dentry->d_inode,
+				  file, cmd, arg);
+		unlock_kernel();
+		return ret;
+	default:
+		/* unknown */
+		printk(KERN_ERR "lirc_dev: %s(%s:%d): Unknown cmd %08x\n",
+		       __func__, current->comm, current->pid, cmd32);
+		return -ENOIOCTLCMD;
+	}
+}
+#endif
+
 static ssize_t irctl_read(struct file *file,
 			  char *buffer,
 			  size_t length,
 			  loff_t *ppos)
 {
-	struct irctl *ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
-	unsigned char buf[ir->buf->chunk_size];
+	struct irctl *ir = irctls[iminor(file->f_dentry->d_inode)];
+	unsigned char buf[ir->chunk_size];
 	int ret = 0, written = 0;
+	int unlock = 1;
 	DECLARE_WAITQUEUE(wait, current);
 
-	dprintk(LOGHEAD "read called\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "read called\n", ir->d.name, ir->d.minor);
 
-	/* if the plugin has a specific read function use it instead */
-	if (ir->p.fops && ir->p.fops->read)
-		return ir->p.fops->read(file, buffer, length, ppos);
+	/* if the driver has a specific read function use it instead */
+	if (ir->d.fops && ir->d.fops->read)
+		return ir->d.fops->read(file, buffer, length, ppos);
 
-	if (down_interruptible(&ir->buffer_sem))
+	if (mutex_lock_interruptible(&ir->buffer_lock))
 		return -ERESTARTSYS;
 	if (!ir->attached) {
-		up(&ir->buffer_sem);
+		mutex_unlock(&ir->buffer_lock);
 		return -ENODEV;
 	}
 
 	if (length % ir->buf->chunk_size) {
 		dprintk(LOGHEAD "read result = -EINVAL\n",
-			ir->p.name, ir->p.minor);
-		up(&ir->buffer_sem);
+			ir->d.name, ir->d.minor);
+		mutex_unlock(&ir->buffer_lock);
 		return -EINVAL;
 	}
 
 	/*
 	 * we add ourselves to the task queue before buffer check
 	 * to avoid losing scan code (in case when queue is awaken somewhere
-	 * beetwen while condition checking and scheduling)
+	 * between while condition checking and scheduling)
 	 */
 	add_wait_queue(&ir->buf->wait_poll, &wait);
 	set_current_state(TASK_INTERRUPTIBLE);
 
 	/*
-	 * while we did't provide 'length' bytes, device is opened in blocking
+	 * while we didn't provide 'length' bytes, device is opened in blocking
 	 * mode and 'copy_to_user' is happy, wait for data.
 	 */
 	while (written < length && ret == 0) {
@@ -805,14 +876,23 @@ static ssize_t irctl_read(struct file *file,
 				ret = -ERESTARTSYS;
 				break;
 			}
+
+			mutex_unlock(&ir->buffer_lock);
 			schedule();
 			set_current_state(TASK_INTERRUPTIBLE);
+
+			if (mutex_lock_interruptible(&ir->buffer_lock)) {
+				unlock = 0;
+				ret = -ERESTARTSYS;
+				break;
+			}
+
 			if (!ir->attached) {
 				ret = -ENODEV;
 				break;
 			}
 		} else {
-			lirc_buffer_read_1(ir->buf, buf);
+			lirc_buffer_read(ir->buf, buf);
 			ret = copy_to_user((void *)buffer+written, buf,
 					   ir->buf->chunk_size);
 			written += ir->buf->chunk_size;
@@ -821,10 +901,10 @@ static ssize_t irctl_read(struct file *file,
 
 	remove_wait_queue(&ir->buf->wait_poll, &wait);
 	set_current_state(TASK_RUNNING);
-	up(&ir->buffer_sem);
+	if(unlock) mutex_unlock(&ir->buffer_lock);
 
 	dprintk(LOGHEAD "read result = %s (%d)\n",
-		ir->p.name, ir->p.minor, ret ? "-EFAULT" : "OK", ret);
+		ir->d.name, ir->d.minor, ret ? "-EFAULT" : "OK", ret);
 
 	return ret ? ret : written;
 }
@@ -837,8 +917,8 @@ void *lirc_get_pdata(struct file *file)
 	if (file && file->f_dentry && file->f_dentry->d_inode &&
 	    file->f_dentry->d_inode->i_rdev) {
 		struct irctl *ir;
-		ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
-		data = ir->p.data;
+		ir = irctls[iminor(file->f_dentry->d_inode)];
+		data = ir->d.data;
 	}
 
 	return data;
@@ -849,13 +929,13 @@ EXPORT_SYMBOL(lirc_get_pdata);
 static ssize_t irctl_write(struct file *file, const char *buffer,
 			   size_t length, loff_t *ppos)
 {
-	struct irctl *ir = &irctls[MINOR(file->f_dentry->d_inode->i_rdev)];
+	struct irctl *ir = irctls[iminor(file->f_dentry->d_inode)];
 
-	dprintk(LOGHEAD "write called\n", ir->p.name, ir->p.minor);
+	dprintk(LOGHEAD "write called\n", ir->d.name, ir->d.minor);
 
-	/* if the plugin has a specific read function use it instead */
-	if (ir->p.fops && ir->p.fops->write)
-		return ir->p.fops->write(file, buffer, length, ppos);
+	/* if the driver has a specific read function use it instead */
+	if (ir->d.fops && ir->d.fops->write)
+		return ir->d.fops->write(file, buffer, length, ppos);
 
 	if (!ir->attached)
 		return -ENODEV;
@@ -865,22 +945,23 @@ static ssize_t irctl_write(struct file *file, const char *buffer,
 
 
 static struct file_operations fops = {
+	.owner		= THIS_MODULE,
 	.read		= irctl_read,
 	.write		= irctl_write,
 	.poll		= irctl_poll,
 	.ioctl		= irctl_ioctl,
+#ifdef CONFIG_COMPAT
+	.compat_ioctl	= irctl_compat_ioctl,
+#endif
 	.open		= irctl_open,
 	.release	= irctl_close
 };
 
+/* For now don't try to use it as a static version !  */
+#ifdef MODULE
 
-static int lirc_dev_init(void)
+static int __init lirc_dev_init(void)
 {
-	int i;
-
-	for (i = 0; i < MAX_IRCTL_DEVICES; ++i)
-		init_irctl(&irctls[i]);
-
 	if (register_chrdev(IRCTL_DEV_MAJOR, IRCTL_DEV_NAME, &fops)) {
 		printk(KERN_ERR "lirc_dev: register_chrdev failed\n");
 		goto out;
@@ -895,7 +976,7 @@ static int lirc_dev_init(void)
 	printk(KERN_INFO "lirc_dev: IR Remote Control driver registered, "
 	       "major %d \n", IRCTL_DEV_MAJOR);
 
-	return SUCCESS;
+	return 0;
 
 out_unregister:
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
@@ -909,24 +990,7 @@ out:
 	return -1;
 }
 
-/* ---------------------------------------------------------------------- */
-
-/* For now dont try to use it as a static version !  */
-
-#ifdef MODULE
-
-/*
- *
- */
-int init_module(void)
-{
-	return lirc_dev_init();
-}
-
-/*
- *
- */
-void cleanup_module(void)
+static void __exit lirc_dev_exit(void)
 {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 23)
 	int ret;
@@ -947,20 +1011,15 @@ void cleanup_module(void)
 #endif
 }
 
+module_init(lirc_dev_init);
+module_exit(lirc_dev_exit);
+
 MODULE_DESCRIPTION("LIRC base driver module");
 MODULE_AUTHOR("Artur Lipowski");
 MODULE_LICENSE("GPL");
 MODULE_ALIAS_CHARDEV_MAJOR(IRCTL_DEV_MAJOR);
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
 #endif /* MODULE */
-
-/*
- * Overrides for Emacs so that we follow Linus's tabbing style.
- * ---------------------------------------------------------------------------
- * Local variables:
- * c-basic-offset: 8
- * End:
- */
