@@ -1,7 +1,15 @@
-/*      $Id: lirc_i2c.c,v 1.46 2008/05/04 13:49:53 lirc Exp $      */
+/*      $Id: lirc_i2c.c,v 1.70 2009/08/30 16:59:53 jarodwilson Exp $      */
 
 /*
- * i2c IR lirc plugin for Hauppauge and Pixelview cards - new 2.3.x i2c stack
+ * lirc_i2c.c
+ *
+ * i2c IR driver for the onboard IR port on many TV tuner cards, including:
+ *  -Flavors of the Hauppauge PVR-150/250/350
+ *  -Hauppauge HVR-1300
+ *  -PixelView (BT878P+W/FM)
+ *  -KNC ONE TV Station/Anubis Typhoon TView Tuner
+ *  -Asus TV-Box and Creative/VisionTek BreakOut-Box
+ *  -Leadtek Winfast PVR2000
  *
  * Copyright (c) 2000 Gerd Knorr <kraxel@goldbach.in-berlin.de>
  * modified for PixelView (BT878P+W/FM) by
@@ -33,7 +41,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
  */
 
 
@@ -54,13 +61,11 @@
 #include <linux/i2c.h>
 #include <linux/i2c-algo-bit.h>
 
-#include <asm/semaphore.h>
-
 #include "drivers/kcompat.h"
 #include "drivers/lirc_dev/lirc_dev.h"
 
 struct IR {
-	struct lirc_plugin l;
+	struct lirc_driver l;
 	struct i2c_client  c;
 	int nextkey;
 	unsigned char b[3];
@@ -68,13 +73,9 @@ struct IR {
 	unsigned char flag;
 };
 
-/* ----------------------------------------------------------------------- */
-
 #define DEVICE_NAME "lirc_i2c"
 
-/* ----------------------------------------------------------------------- */
-/* insmod parameters						       */
-
+/* module parameters */
 static int debug;	/* debug output */
 static int minor = -1;	/* minor number */
 
@@ -85,15 +86,13 @@ static int minor = -1;	/* minor number */
 			       ## args);				\
 	} while (0)
 
-/* ----------------------------------------------------------------------- */
-
-static inline int reverse(int data, int bits)
+static int reverse(int data, int bits)
 {
 	int i;
 	int c;
 
 	for (c = 0, i = 0; i < bits; i++)
-		c |= (((data & (1<<i)) ? 1:0)) << (bits-1-i);
+		c |= ((data & (1<<i)) ? 1 : 0) << (bits-1-i);
 
 	return c;
 }
@@ -122,7 +121,7 @@ static int add_to_buf_adap(void *data, struct lirc_buffer *buf)
 	keybuf[2] &= 0x7f;
 	keybuf[3] |= 0x80;
 
-	lirc_buffer_write_1(buf, keybuf);
+	lirc_buffer_write(buf, keybuf);
 	return 0;
 }
 
@@ -159,14 +158,13 @@ static int add_to_buf_pcf8574(void *data, struct lirc_buffer *buf)
 	dprintk("%s key 0x%02X %s\n", ir->c.name, rc & ir->bits,
 		(rc & ir->flag) ? "released" : "pressed");
 
-	if (rc & ir->flag) {
-		/* ignore released buttons */
+	/* ignore released buttons */
+	if (rc & ir->flag)
 		return -ENODATA;
-	}
 
 	/* set valid key code */
 	key  = rc & ir->bits;
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -177,16 +175,19 @@ static int add_to_buf_haup_common(void *data, struct lirc_buffer *buf,
 	struct IR *ir = data;
 	__u16 code;
 	unsigned char codes[2];
+	int ret;
 
 	/* poll IR chip */
-	if (size == i2c_master_recv(&ir->c, keybuf, size)) {
+	ret = i2c_master_recv(&ir->c, keybuf, size);
+	if (ret == size) {
 		ir->b[0] = keybuf[offset];
 		ir->b[1] = keybuf[offset+1];
 		ir->b[2] = keybuf[offset+2];
-		dprintk("key (0x%02x/0x%02x)\n", ir->b[0], ir->b[1]);
+		if (ir->b[0] != 0x00 && ir->b[1] != 0x00)
+			dprintk("key (0x%02x/0x%02x)\n", ir->b[0], ir->b[1]);
 	} else {
-		dprintk("read error\n");
-		/* keep last successfull read buffer */
+		dprintk("read error (ret=%d)\n", ret);
+		/* keep last successful read buffer */
 	}
 
 	/* key pressed ? */
@@ -200,7 +201,8 @@ static int add_to_buf_haup_common(void *data, struct lirc_buffer *buf,
 	codes[1] = code & 0xff;
 
 	/* return it */
-	lirc_buffer_write_1(buf, codes);
+	dprintk("sending code 0x%02x%02x to lirc\n", codes[0], codes[1]);
+	lirc_buffer_write(buf, codes);
 	return 0;
 }
 
@@ -250,7 +252,7 @@ static int add_to_buf_pvr2000(void *data, struct lirc_buffer *buf)
 	dprintk("IR Key/Flags: (0x%02x/0x%02x)\n", key, flags & 0xFF);
 
 	/* return it */
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -267,7 +269,7 @@ static int add_to_buf_pixelview(void *data, struct lirc_buffer *buf)
 	dprintk("key %02x\n", key);
 
 	/* return it */
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 	return 0;
 }
 
@@ -292,7 +294,7 @@ static int add_to_buf_pv951(void *data, struct lirc_buffer *buf)
 	codes[2] = reverse(key, 8);
 	codes[3] = (~codes[2])&0xff;
 
-	lirc_buffer_write_1(buf, codes);
+	lirc_buffer_write(buf, codes);
 	return 0;
 }
 
@@ -308,9 +310,11 @@ static int add_to_buf_knc1(void *data, struct lirc_buffer *buf)
 		return -ENODATA;
 	}
 
-	/* it seems that 0xFE indicates that a button is still hold
-	   down, while 0xFF indicates that no button is hold
-	   down. 0xFE sequences are sometimes interrupted by 0xFF */
+	/*
+	 * it seems that 0xFE indicates that a button is still held
+	 * down, while 0xFF indicates that no button is held
+	 * down. 0xFE sequences are sometimes interrupted by 0xFF
+	 */
 
 	dprintk("key %02x\n", key);
 
@@ -321,7 +325,7 @@ static int add_to_buf_knc1(void *data, struct lirc_buffer *buf)
 		key = last_key;
 
 	last_key = key;
-	lirc_buffer_write_1(buf, &key);
+	lirc_buffer_write(buf, &key);
 
 	return 0;
 }
@@ -329,12 +333,15 @@ static int add_to_buf_knc1(void *data, struct lirc_buffer *buf)
 static int set_use_inc(void *data)
 {
 	struct IR *ir = data;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 25)
+	int ret;
+#endif
+
+	dprintk("%s called\n", __func__);
 
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 25)
 	i2c_use_client(&ir->c);
 #else
-	int ret;
-
 	/* lock bttv in memory while /dev/lirc is in use  */
 	ret = i2c_use_client(&ir->c);
 	if (ret != 0)
@@ -349,11 +356,13 @@ static void set_use_dec(void *data)
 {
 	struct IR *ir = data;
 
+	dprintk("%s called\n", __func__);
+
 	i2c_release_client(&ir->c);
 	MOD_DEC_USE_COUNT;
 }
 
-static struct lirc_plugin lirc_template = {
+static struct lirc_driver lirc_template = {
 	.name		= "lirc_i2c",
 	.set_use_inc	= set_use_inc,
 	.set_use_dec	= set_use_dec,
@@ -361,14 +370,25 @@ static struct lirc_plugin lirc_template = {
 	.owner		= THIS_MODULE,
 };
 
-/* ----------------------------------------------------------------------- */
-
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 static int ir_attach(struct i2c_adapter *adap, int addr,
 		      unsigned short flags, int kind);
-static int ir_detach(struct i2c_client *client);
 static int ir_probe(struct i2c_adapter *adap);
+# else
+static int ir_probe(struct i2c_client *client, const struct i2c_device_id *id);
+#endif
+static int ir_remove(struct i2c_client *client);
 static int ir_command(struct i2c_client *client, unsigned int cmd, void *arg);
 
+#if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 30)
+static const struct i2c_device_id ir_receiver_id[] = {
+	/* Generic entry for any IR receiver */
+	{ "ir_video", 0 },
+	/* IR device specific entries could be added here */
+	{ }
+};
+#endif
+
 static struct i2c_driver driver = {
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 16)
 	.name		= "i2c ir driver",
@@ -380,37 +400,84 @@ static struct i2c_driver driver = {
 	},
 #endif
 	.id		= I2C_DRIVERID_EXP3, /* FIXME */
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 	.attach_adapter	= ir_probe,
-	.detach_client	= ir_detach,
+	.detach_client	= ir_remove,
+#else
+	.probe		= ir_probe,
+	.remove		= ir_remove,
+	.id_table	= ir_receiver_id,
+#endif
 	.command	= ir_command,
 };
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 static struct i2c_client client_template = {
 	.name		= "unset",
 	.driver		= &driver
 };
+#endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 static int ir_attach(struct i2c_adapter *adap, int addr,
 		     unsigned short flags, int kind)
+#else
+static void pcf_probe(struct i2c_client *client, struct IR *ir)
+{
+	int ret1, ret2, ret3, ret4;
+
+	ret1 = i2c_smbus_write_byte(client, 0xff);
+	ret2 = i2c_smbus_read_byte(client);
+	ret3 = i2c_smbus_write_byte(client, 0x00);
+	ret4 = i2c_smbus_read_byte(client);
+
+	/* in the Asus TV-Box: bit 1-0 */
+	if (((ret2 & 0x03) == 0x03) && ((ret4 & 0x03) == 0x00)) {
+		ir->bits = (unsigned char) ~0x07;
+		ir->flag = 0x04;
+	/* in the Creative/VisionTek BreakOut-Box: bit 7-6 */
+	} else if (((ret2 & 0xc0) == 0xc0) && ((ret4 & 0xc0) == 0x00)) {
+		ir->bits = (unsigned char) ~0xe0;
+		ir->flag = 0x20;
+	}
+
+	return;
+}
+
+static int ir_probe(struct i2c_client *client, const struct i2c_device_id *id)
+#endif
 {
 	struct IR *ir;
-	int err;
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+	int err, retval;
 
 	client_template.adapter = adap;
 	client_template.addr = addr;
+#else
+	struct i2c_adapter *adap = client->adapter;
+	unsigned short addr = client->addr;
+	int retval;
+#endif
 
 	ir = kmalloc(sizeof(struct IR), GFP_KERNEL);
 	if (!ir)
 		return -ENOMEM;
-	memcpy(&ir->l, &lirc_template, sizeof(struct lirc_plugin));
+	memcpy(&ir->l, &lirc_template, sizeof(struct lirc_driver));
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 	memcpy(&ir->c, &client_template, sizeof(struct i2c_client));
 
 	ir->c.adapter = adap;
 	ir->c.addr    = addr;
 	i2c_set_clientdata(&ir->c, ir);
+#else
+	memcpy(&ir->c, client, sizeof(struct i2c_client));
+
+	i2c_set_clientdata(client, ir);
+#endif
 	ir->l.data    = ir;
 	ir->l.minor   = minor;
 	ir->l.sample_rate = 10;
+	ir->l.dev     = &ir->c.dev;
 	ir->nextkey   = -1;
 
 	switch (addr) {
@@ -431,9 +498,11 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 #else
 		if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848)) {
 #endif
-			/* The PVR150 IR receiver uses the same protocol as
+			/*
+			 * The PVR150 IR receiver uses the same protocol as
 			 * other Hauppauge cards, but the data flow is
-			 * different, so we need to deal with it by its own. */
+			 * different, so we need to deal with it by its own.
+			 */
 			strlcpy(ir->c.name, "Hauppauge PVR150", I2C_NAME_SIZE);
 		} else /* I2C_HW_B_CX2388x */
 			strlcpy(ir->c.name, "Hauppauge HVR1300", I2C_NAME_SIZE);
@@ -469,48 +538,70 @@ static int ir_attach(struct i2c_adapter *adap, int addr,
 		break;
 	case 0x21:
 	case 0x23:
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+		ir->bits = flags & 0xff;
+		ir->flag = (flags >> 8) & 0xff;
+#else
+		pcf_probe(client, ir);
+#endif
 		strlcpy(ir->c.name, "TV-Box IR", I2C_NAME_SIZE);
 		ir->l.code_length = 8;
 		ir->l.add_to_buf = add_to_buf_pcf8574;
-		ir->bits = flags & 0xff;
-		ir->flag = (flags >> 8) & 0xff;
 		break;
 	default:
 		/* shouldn't happen */
 		printk("lirc_i2c: Huh? unknown i2c address (0x%02x)?\n", addr);
 		kfree(ir);
-		return -1;
+		return -EINVAL;
 	}
 	printk(KERN_INFO "lirc_i2c: chip 0x%x found @ 0x%02x (%s)\n",
 	       adap->id, addr, ir->c.name);
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 	/* register device */
 	err = i2c_attach_client(&ir->c);
 	if (err) {
 		kfree(ir);
 		return err;
 	}
-	ir->l.minor = lirc_register_plugin(&ir->l);
+#endif
+
+	retval = lirc_register_driver(&ir->l);
+
+	if (retval < 0) {
+		printk(KERN_ERR "lirc_i2c: failed to register driver!\n");
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
+		i2c_detach_client(&ir->c);
+#endif
+		kfree(ir);
+		return retval;
+	}
+
+	ir->l.minor = retval;
 
 	return 0;
 }
 
-static int ir_detach(struct i2c_client *client)
+static int ir_remove(struct i2c_client *client)
 {
 	struct IR *ir = i2c_get_clientdata(client);
 
 	/* unregister device */
-	lirc_unregister_plugin(ir->l.minor);
+	lirc_unregister_driver(ir->l.minor);
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 	i2c_detach_client(&ir->c);
+#endif
 
 	/* free memory */
 	kfree(ir);
 	return 0;
 }
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 31)
 static int ir_probe(struct i2c_adapter *adap)
 {
-	/* The external IR receiver is at i2c address 0x34 (0x35 for
+	/*
+	 * The external IR receiver is at i2c address 0x34 (0x35 for
 	 * reads).  Future Hauppauge cards will have an internal
 	 * receiver at 0x30 (0x31 for reads).  In theory, both can be
 	 * fitted, and Hauppauge suggest an external overrides an
@@ -519,7 +610,8 @@ static int ir_probe(struct i2c_adapter *adap)
 	 * That's why we probe 0x1a (~0x34) first. CB
 	 *
 	 * The i2c address for the Hauppauge PVR-150 card is 0xe2,
-	 * so we need to probe 0x71 as well. */
+	 * so we need to probe 0x71 as well.
+	 */
 
 	static const int probe[] = {
 		0x1a, /* Hauppauge IR external */
@@ -542,14 +634,13 @@ static int ir_probe(struct i2c_adapter *adap)
 	char buf;
 	int i, rc;
 
+	memset(&c, 0, sizeof(c));
 #ifdef I2C_HW_B_CX2341X
 	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848) ||
-	    adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2341X))
+	    adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2341X)) {
 #else
-	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848))
+	if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_BT848)) {
 #endif
-	{
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != probe[i]; i++) {
 			c.addr = probe[i];
@@ -557,15 +648,17 @@ static int ir_probe(struct i2c_adapter *adap)
 			dprintk("probe 0x%02x @ %s: %s\n",
 				probe[i], adap->name,
 				(1 == rc) ? "yes" : "no");
-			if (1 == rc)
-				ir_attach(adap, probe[i], 0, 0);
+			if (1 == rc) {
+				rc = ir_attach(adap, probe[i], 0, 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 
 #ifdef I2C_HW_B_CX2388x
 	/* Leadtek Winfast PVR2000 or Hauppauge HVR-1300 */
 	else if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_CX2388x)) {
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != probe_cx88[i]; i++) {
 			c.addr = probe_cx88[i];
@@ -573,27 +666,30 @@ static int ir_probe(struct i2c_adapter *adap)
 			dprintk("probe 0x%02x @ %s: %s\n",
 				c.addr, adap->name,
 				(1 == rc) ? "yes" : "no");
-			if (1 == rc)
-				ir_attach(adap, c.addr, 0, 0);
+			if (1 == rc) {
+				rc = ir_attach(adap, c.addr, 0, 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 #endif
 
 	/* Asus TV-Box and Creative/VisionTek BreakOut-Box (PCF8574) */
 	else if (adap->id == (I2C_ALGO_BIT | I2C_HW_B_RIVA)) {
-		/* addresses to probe;
-		   leave 0x24 and 0x25 because SAA7113H possibly uses it
-		   0x21 and 0x22 possibly used by SAA7108E
-		   Asus:      0x21 is a correct address (channel 1 of PCF8574)
-		   Creative:  0x23 is a correct address (channel 3 of PCF8574)
-		   VisionTek: 0x23 is a correct address (channel 3 of PCF8574)
-		*/
+		/*
+		 * addresses to probe;
+		 * leave 0x24 and 0x25 because SAA7113H possibly uses it
+		 * 0x21 and 0x22 possibly used by SAA7108E
+		 * Asus:      0x21 is a correct address (channel 1 of PCF8574)
+		 * Creative:  0x23 is a correct address (channel 3 of PCF8574)
+		 * VisionTek: 0x23 is a correct address (channel 3 of PCF8574)
+		 */
 		static const int pcf_probe[] = { 0x20, 0x21, 0x22, 0x23,
 						 0x24, 0x25, 0x26, 0x27, -1 };
 		int ret1, ret2, ret3, ret4;
 		unsigned char bits = 0, flag = 0;
 
-		memset(&c, 0, sizeof(c));
 		c.adapter = adap;
 		for (i = 0; -1 != pcf_probe[i]; i++) {
 			c.addr = pcf_probe[i];
@@ -623,14 +719,23 @@ static int ir_probe(struct i2c_adapter *adap)
 			}
 			dprintk("probe 0x%02x @ %s: %s\n",
 				c.addr, adap->name, rc ? "yes" : "no");
-			if (rc)
-				ir_attach(adap, pcf_probe[i],
-					  bits|(flag<<8), 0);
+			if (rc) {
+				rc = ir_attach(adap, pcf_probe[i],
+					       bits | (flag << 8), 0);
+				if (rc < 0)
+					goto attach_fail;
+			}
 		}
 	}
 
 	return 0;
+
+attach_fail:
+	printk(KERN_ERR "lirc_i2c: %s: ir_attach failed!\n", __func__);
+	return rc;
+
 }
+#endif
 
 static int ir_command(struct i2c_client *client, unsigned int cmd, void *arg)
 {
@@ -638,10 +743,9 @@ static int ir_command(struct i2c_client *client, unsigned int cmd, void *arg)
 	return 0;
 }
 
-/* ----------------------------------------------------------------------- */
 #ifdef MODULE
 
-int init_module(void)
+static int __init lirc_i2c_init(void)
 {
 	request_module("bttv");
 	request_module("rivatv");
@@ -651,7 +755,7 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit lirc_i2c_exit(void)
 {
 	i2c_del_driver(&driver);
 }
@@ -662,20 +766,14 @@ MODULE_AUTHOR("Gerd Knorr, Michal Kochanowicz, Christoph Bartelmus, "
 	      "Ulrich Mueller, Stefan Jahn, Jerome Brock");
 MODULE_LICENSE("GPL");
 
-module_param(minor, int, 0444);
+module_param(minor, int, S_IRUGO);
 MODULE_PARM_DESC(minor, "Preferred minor device number");
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
+module_init(lirc_i2c_init);
+module_exit(lirc_i2c_exit);
 EXPORT_NO_SYMBOLS;
 
 #endif /* MODULE */
-
-/*
- * Overrides for Emacs so that we follow Linus's tabbing style.
- * ---------------------------------------------------------------------------
- * Local variables:
- * c-basic-offset: 8
- * End:
- */
