@@ -1,8 +1,6 @@
-/*      $Id: lirc_parallel.c,v 5.40 2008/01/13 11:13:50 lirc Exp $      */
-
-/****************************************************************************
- ** lirc_parallel.c *********************************************************
- ****************************************************************************
+/*      $Id: lirc_parallel.c,v 5.53 2009/03/08 19:22:28 lirc Exp $      */
+/*
+ * lirc_parallel.c
  *
  * lirc_parallel - device driver for infra-red signal receiving and
  *                 transmitting unit built by the author
@@ -25,14 +23,10 @@
  *
  */
 
-/***********************************************************************
- *************************       Includes        ***********************
- ***********************************************************************/
-
+/*** Includes ***/
 #ifdef HAVE_CONFIG_H
 # include <config.h>
 #endif
-
 #include <linux/version.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 2, 18)
 #error "**********************************************************"
@@ -41,7 +35,6 @@
 #endif
 
 #include <linux/autoconf.h>
-
 #ifdef CONFIG_SMP
 #error "--- Sorry, this driver is not SMP safe. ---"
 #endif
@@ -94,9 +87,7 @@
 #define LIRC_TIMER 65536
 #endif
 
-/***********************************************************************
- *************************   Globale Variablen   ***********************
- ***********************************************************************/
+/*** Global Variables ***/
 
 static int debug;
 static int check_pselecd;
@@ -127,24 +118,22 @@ int is_claimed;
 
 unsigned int tx_mask = 1;
 
-/***********************************************************************
- *************************   Interne Funktionen  ***********************
- ***********************************************************************/
+/*** Internal Functions ***/
 
-static inline unsigned int in(int offset)
+static unsigned int in(int offset)
 {
 	switch (offset) {
 	case LIRC_LP_BASE:
-		return (parport_read_data(pport));
+		return parport_read_data(pport);
 	case LIRC_LP_STATUS:
-		return (parport_read_status(pport));
+		return parport_read_status(pport);
 	case LIRC_LP_CONTROL:
-		return (parport_read_control(pport));
+		return parport_read_control(pport);
 	}
 	return 0; /* make compiler happy */
 }
 
-static inline void out(int offset, int value)
+static void out(int offset, int value)
 {
 	switch (offset) {
 	case LIRC_LP_BASE:
@@ -160,22 +149,22 @@ static inline void out(int offset, int value)
 	}
 }
 
-static inline unsigned int lirc_get_timer(void)
+static unsigned int lirc_get_timer(void)
 {
-	return (in(LIRC_PORT_TIMER)&LIRC_PORT_TIMER_BIT);
+	return in(LIRC_PORT_TIMER) & LIRC_PORT_TIMER_BIT;
 }
 
-static inline  unsigned int lirc_get_signal(void)
+static unsigned int lirc_get_signal(void)
 {
-	return (in(LIRC_PORT_SIGNAL)&LIRC_PORT_SIGNAL_BIT);
+	return in(LIRC_PORT_SIGNAL) & LIRC_PORT_SIGNAL_BIT;
 }
 
-static inline void lirc_on(void)
+static void lirc_on(void)
 {
 	out(LIRC_PORT_DATA, tx_mask);
 }
 
-static inline void lirc_off(void)
+static void lirc_off(void)
 {
 	out(LIRC_PORT_DATA, 0);
 }
@@ -248,11 +237,9 @@ static int lirc_claim(void)
 	return 1;
 }
 
-/***********************************************************************
- *************************   interrupt handler  ************************
- ***********************************************************************/
+/*** interrupt handler ***/
 
-static inline void rbuf_write(lirc_t signal)
+static void rbuf_write(lirc_t signal)
 {
 	unsigned int nwptr;
 
@@ -269,8 +256,10 @@ static inline void rbuf_write(lirc_t signal)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 19)
 static void irq_handler(int i, void *blah, struct pt_regs *regs)
-#else
+#elif LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 24)
 static void irq_handler(int i, void *blah)
+#else
+static void irq_handler(void *blah)
 #endif
 {
 	struct timeval tv;
@@ -287,11 +276,11 @@ static void irq_handler(int i, void *blah)
 	if (!is_claimed)
 		return;
 
+#if 0
 	/* disable interrupt */
-	/*
-	  disable_irq(irq);
-	  out(LIRC_PORT_IRQ, in(LIRC_PORT_IRQ) & (~LP_PINTEN));
-	*/
+	disable_irq(irq);
+	out(LIRC_PORT_IRQ, in(LIRC_PORT_IRQ) & (~LP_PINTEN));
+#endif
 	if (check_pselecd && (in(1) & LP_PSELECD))
 		return;
 
@@ -311,9 +300,10 @@ static void irq_handler(int i, void *blah)
 		rbuf_write(data); /* space */
 	} else {
 		if (timer == 0) {
-			/* wake up; we'll lose this signal
-			 * but it will be garbage if the device
-			 * is turned on anyway */
+			/*
+			 * wake up; we'll lose this signal, but it will be
+			 * garbage if the device is turned on anyway
+			 */
 			timer = init_lirc_timer();
 			/* enable_irq(irq); */
 			return;
@@ -337,10 +327,10 @@ static void irq_handler(int i, void *blah)
 			printk(KERN_NOTICE "%s: timeout\n", LIRC_DRIVER_NAME);
 			break;
 		}
-	}
-	while (lirc_get_signal());
+	} while (lirc_get_signal());
+
 	if (signal != 0) {
-		/* ajust value to usecs */
+		/* adjust value to usecs */
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
 		unsigned long long helper;
 
@@ -371,9 +361,7 @@ static void irq_handler(int i, void *blah)
 	*/
 }
 
-/***********************************************************************
- **************************   file_operations   ************************
- ***********************************************************************/
+/*** file operations ***/
 
 static loff_t lirc_lseek(struct file *filep, loff_t offset, int orig)
 {
@@ -415,7 +403,7 @@ static ssize_t lirc_read(struct file *filep, char *buf, size_t n, loff_t *ppos)
 	}
 	remove_wait_queue(&lirc_wait, &wait);
 	set_current_state(TASK_RUNNING);
-	return (count ? count : result);
+	return count ? count : result;
 }
 
 static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
@@ -428,15 +416,15 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 	lirc_t counttimer;
 
 	if (!is_claimed)
-		return(-EBUSY);
+		return -EBUSY;
 
 	if (n % sizeof(lirc_t))
-		return(-EINVAL);
+		return -EINVAL;
 
 	count = n / sizeof(lirc_t);
 
 	if (count > WBUF_SIZE || count % 2 == 0)
-		return(-EINVAL);
+		return -EINVAL;
 
 	if (copy_from_user(wbuf, buf, n))
 		return -EFAULT;
@@ -446,10 +434,10 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 		/* try again if device is ready */
 		timer = init_lirc_timer();
 		if (timer == 0)
-			return(-EIO);
+			return -EIO;
 	}
 
-	/* ajust values from usecs */
+	/* adjust values from usecs */
 	for (i = 0; i < count; i++) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 4, 0)
 		unsigned long long helper;
@@ -499,8 +487,7 @@ static ssize_t lirc_write(struct file *filep, const char *buf, size_t n,
 	}
 	local_irq_restore(flags);
 #else
-	/* place code that handles write
-	 * without external timer here */
+	/* place code that handles write without external timer here */
 #endif
 	return n;
 }
@@ -509,7 +496,7 @@ static unsigned int lirc_poll(struct file *file, poll_table *wait)
 {
 	poll_wait(file, &lirc_wait, wait);
 	if (rptr != wptr)
-		return (POLLIN|POLLRDNORM);
+		return POLLIN | POLLRDNORM;
 	return 0;
 }
 
@@ -595,6 +582,7 @@ static int lirc_close(struct inode *node, struct file *filep)
 }
 
 static struct file_operations lirc_fops = {
+	.owner		= THIS_MODULE,
 	.llseek		= lirc_lseek,
 	.read		= lirc_read,
 	.write		= lirc_write,
@@ -606,27 +594,23 @@ static struct file_operations lirc_fops = {
 
 static int set_use_inc(void *data)
 {
-#if WE_DONT_USE_LOCAL_OPEN_CLOSE
-	MOD_INC_USE_COUNT;
-#endif
 	return 0;
 }
 
 static void set_use_dec(void *data)
 {
-#if WE_DONT_USE_LOCAL_OPEN_CLOSE
-	MOD_DEC_USE_COUNT;
-#endif
 }
 
-static struct lirc_plugin plugin = {
+static struct lirc_driver driver = {
        .name		= LIRC_DRIVER_NAME,
        .minor		= -1,
        .code_length	= 1,
        .sample_rate	= 0,
        .data		= NULL,
        .add_to_buf	= NULL,
+#ifndef LIRC_REMOVE_DURING_EXPORT
        .get_queue	= NULL,
+#endif
        .set_use_inc	= set_use_inc,
        .set_use_dec	= set_use_dec,
        .fops		= &lirc_fops,
@@ -635,7 +619,6 @@ static struct lirc_plugin plugin = {
 };
 
 #ifdef MODULE
-
 static int pf(void *handle);
 static void kf(void *handle);
 
@@ -682,11 +665,9 @@ static void kf(void *handle)
 	*/
 }
 
-/***********************************************************************
- ******************   init_module()/cleanup_module()  ******************
- ***********************************************************************/
+/*** module initialization and cleanup ***/
 
-int init_module(void)
+static int __init lirc_parallel_init(void)
 {
 #if LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 3)
 	pport = parport_find_base(io);
@@ -740,8 +721,8 @@ int init_module(void)
 	is_claimed = 0;
 	parport_release(ppdevice);
  skip_init:
-	plugin.minor = lirc_register_plugin(&plugin);
-	if (plugin.minor < 0) {
+	driver.minor = lirc_register_driver(&driver);
+	if (driver.minor < 0) {
 		printk(KERN_NOTICE "%s: register_chrdev() failed\n",
 		       LIRC_DRIVER_NAME);
 		parport_unregister_device(ppdevice);
@@ -752,31 +733,32 @@ int init_module(void)
 	return 0;
 }
 
-void cleanup_module(void)
+static void __exit lirc_parallel_exit(void)
 {
 	parport_unregister_device(ppdevice);
-	lirc_unregister_plugin(plugin.minor);
+	lirc_unregister_driver(driver.minor);
 }
 
+module_init(lirc_parallel_init);
+module_exit(lirc_parallel_exit);
+
 MODULE_DESCRIPTION("Infrared receiver driver for parallel ports.");
 MODULE_AUTHOR("Christoph Bartelmus");
 MODULE_LICENSE("GPL");
 
-module_param(io, int, 0444);
+module_param(io, int, S_IRUGO);
 MODULE_PARM_DESC(io, "I/O address base (0x3bc, 0x378 or 0x278)");
 
-module_param(irq, int, 0444);
+module_param(irq, int, S_IRUGO);
 MODULE_PARM_DESC(irq, "Interrupt (7 or 5)");
 
-module_param(tx_mask, int, 0444);
+module_param(tx_mask, int, S_IRUGO);
 MODULE_PARM_DESC(tx_maxk, "Transmitter mask (default: 0x01)");
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
 
-module_param(check_pselecd, bool, 0644);
+module_param(check_pselecd, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Check for printer (default: 0)");
-
 EXPORT_NO_SYMBOLS;
-
 #endif /* MODULE */
