@@ -1,5 +1,4 @@
-/*      $Id: lirc_streamzap.c,v 1.27 2008/01/13 11:13:50 lirc Exp $      */
-
+/*      $Id: lirc_streamzap.c,v 1.48 2009/03/15 09:34:00 lirc Exp $      */
 /*
  * Streamzap Remote Control driver
  *
@@ -28,7 +27,6 @@
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
- *
  */
 
 #include <linux/version.h>
@@ -37,7 +35,6 @@
 #error "Sorry, this driver needs kernel version 2.4.0 or higher"
 #error "*******************************************************"
 #endif
-
 #include <linux/autoconf.h>
 #include <linux/kernel.h>
 #include <linux/errno.h>
@@ -57,12 +54,10 @@
 #include "drivers/kcompat.h"
 #include "drivers/lirc_dev/lirc_dev.h"
 
-#define DRIVER_VERSION	"$Revision: 1.27 $"
+#define DRIVER_VERSION	"$Revision: 1.48 $"
 #define DRIVER_NAME	"lirc_streamzap"
 #define DRIVER_DESC	"Streamzap Remote Control driver"
 
-/* ------------------------------------------------------------------ */
-
 static int debug;
 
 #define USB_STREAMZAP_VENDOR_ID		0x0e9c
@@ -76,12 +71,12 @@ static int debug;
 			       fmt "\n", ## args);		\
 	} while (0)
 
-/*
- * table of devices that work with this driver
- */
-static struct usb_device_id streamzap_table [] = {
+/* table of devices that work with this driver */
+static struct usb_device_id streamzap_table[] = {
+	/* Streamzap Remote Control */
 	{ USB_DEVICE(USB_STREAMZAP_VENDOR_ID, USB_STREAMZAP_PRODUCT_ID) },
-	{ }	/* Terminating entry */
+	/* Terminating entry */
+	{ }
 };
 
 MODULE_DEVICE_TABLE(usb, streamzap_table);
@@ -100,25 +95,26 @@ enum StreamzapDecoderState {
 	IgnorePulse
 };
 
-/* Structure to hold all of our device specific stuff */
-/* some remarks regarding locking:
-   theoretically this struct can be accessed from three threads:
-
-   - from lirc_dev through set_use_inc/set_use_dec
-
-   - from the USB layer throuh probe/disconnect/irq
-
-     Careful placement of lirc_register_plugin/lirc_unregister_plugin
-     calls will prevent conflicts. lirc_dev makes sure that
-     set_use_inc/set_use_dec are not being executed and will not be
-     called after lirc_unregister_plugin returns.
-
-   - by the timer callback
-
-     The timer is only running when the device is connected and the
-     LIRC device is open. Making sure the timer is deleted by
-     set_use_dec will make conflicts impossible.
-*/
+/* Structure to hold all of our device specific stuff
+ *
+ * some remarks regarding locking:
+ * theoretically this struct can be accessed from three threads:
+ *
+ * - from lirc_dev through set_use_inc/set_use_dec
+ *
+ * - from the USB layer throuh probe/disconnect/irq
+ *
+ *   Careful placement of lirc_register_driver/lirc_unregister_driver
+ *   calls will prevent conflicts. lirc_dev makes sure that
+ *   set_use_inc/set_use_dec are not being executed and will not be
+ *   called after lirc_unregister_driver returns.
+ *
+ * - by the timer callback
+ *
+ *   The timer is only running when the device is connected and the
+ *   LIRC device is open. Making sure the timer is deleted by
+ *   set_use_dec will make conflicts impossible.
+ */
 struct usb_streamzap {
 
 	/* usb */
@@ -138,7 +134,7 @@ struct usb_streamzap {
 	struct urb		*urb_in;
 
 	/* lirc */
-	struct lirc_plugin	plugin;
+	struct lirc_driver	driver;
 	struct lirc_buffer	delay_buf;
 	struct lirc_buffer	lirc_buf;
 
@@ -207,9 +203,11 @@ static void stop_timer(struct usb_streamzap *sz)
 	spin_lock_irqsave(&sz->timer_lock, flags);
 	if (sz->timer_running) {
 		sz->timer_running = 0;
+		spin_unlock_irqrestore(&sz->timer_lock, flags);
 		del_timer_sync(&sz->delay_timer);
+	} else {
+		spin_unlock_irqrestore(&sz->timer_lock, flags);
 	}
-	spin_unlock_irqrestore(&sz->timer_lock, flags);
 }
 
 static void flush_timeout(unsigned long arg)
@@ -224,7 +222,7 @@ static void delay_timeout(unsigned long arg)
 	unsigned long flags;
 	/* deliver data every 10 ms */
 	static unsigned long timer_inc =
-		(10000/(1000000/HZ)) == 0 ? 1:(10000/(1000000/HZ));
+		(10000/(1000000/HZ)) == 0 ? 1 : (10000/(1000000/HZ));
 	struct usb_streamzap *sz = (struct usb_streamzap *) arg;
 	lirc_t data;
 
@@ -232,20 +230,20 @@ static void delay_timeout(unsigned long arg)
 
 	if (!lirc_buffer_empty(&sz->delay_buf) &&
 	    !lirc_buffer_full(&sz->lirc_buf)) {
-		lirc_buffer_read_1(&sz->delay_buf, (unsigned char *) &data);
-		lirc_buffer_write_1(&sz->lirc_buf, (unsigned char *) &data);
+		lirc_buffer_read(&sz->delay_buf, (unsigned char *) &data);
+		lirc_buffer_write(&sz->lirc_buf, (unsigned char *) &data);
 	}
 	if (!lirc_buffer_empty(&sz->delay_buf)) {
 		while (lirc_buffer_available(&sz->delay_buf) <
 		      STREAMZAP_BUFFER_SIZE/2 &&
 		      !lirc_buffer_full(&sz->lirc_buf)) {
-			lirc_buffer_read_1(&sz->delay_buf,
-					   (unsigned char *) &data);
-			lirc_buffer_write_1(&sz->lirc_buf,
-					    (unsigned char *) &data);
+			lirc_buffer_read(&sz->delay_buf,
+					 (unsigned char *) &data);
+			lirc_buffer_write(&sz->lirc_buf,
+					  (unsigned char *) &data);
 		}
 		if (sz->timer_running) {
-			sz->delay_timer.expires += timer_inc;
+			sz->delay_timer.expires = jiffies + timer_inc;
 			add_timer(&sz->delay_timer);
 		}
 	} else {
@@ -258,26 +256,26 @@ static void delay_timeout(unsigned long arg)
 	spin_unlock_irqrestore(&sz->timer_lock, flags);
 }
 
-static inline void flush_delay_buffer(struct usb_streamzap *sz)
+static void flush_delay_buffer(struct usb_streamzap *sz)
 {
 	lirc_t data;
 	int empty = 1;
 
 	while (!lirc_buffer_empty(&sz->delay_buf)) {
 		empty = 0;
-		lirc_buffer_read_1(&sz->delay_buf, (unsigned char *) &data);
+		lirc_buffer_read(&sz->delay_buf, (unsigned char *) &data);
 		if (!lirc_buffer_full(&sz->lirc_buf)) {
-			lirc_buffer_write_1(&sz->lirc_buf,
+			lirc_buffer_write(&sz->lirc_buf,
 					    (unsigned char *) &data);
 		} else {
-			dprintk("buffer overflow\n", sz->plugin.minor);
+			dprintk("buffer overflow", sz->driver.minor);
 		}
 	}
 	if (!empty)
 		wake_up(&sz->lirc_buf.wait_poll);
 }
 
-static inline void push(struct usb_streamzap *sz, unsigned char *data)
+static void push(struct usb_streamzap *sz, unsigned char *data)
 {
 	unsigned long flags;
 
@@ -285,16 +283,16 @@ static inline void push(struct usb_streamzap *sz, unsigned char *data)
 	if (lirc_buffer_full(&sz->delay_buf)) {
 		lirc_t data;
 
-		lirc_buffer_read_1(&sz->delay_buf, (unsigned char *) &data);
+		lirc_buffer_read(&sz->delay_buf, (unsigned char *) &data);
 		if (!lirc_buffer_full(&sz->lirc_buf)) {
-			lirc_buffer_write_1(&sz->lirc_buf,
-					    (unsigned char *) &data);
+			lirc_buffer_write(&sz->lirc_buf,
+					  (unsigned char *) &data);
 		} else {
-			dprintk("buffer overflow", sz->plugin.minor);
+			dprintk("buffer overflow", sz->driver.minor);
 		}
 	}
 
-	lirc_buffer_write_1(&sz->delay_buf, data);
+	lirc_buffer_write(&sz->delay_buf, data);
 
 	if (!sz->timer_running) {
 		sz->delay_timer.expires = jiffies + HZ/10;
@@ -305,7 +303,7 @@ static inline void push(struct usb_streamzap *sz, unsigned char *data)
 	spin_unlock_irqrestore(&sz->timer_lock, flags);
 }
 
-static inline void push_full_pulse(struct usb_streamzap *sz,
+static void push_full_pulse(struct usb_streamzap *sz,
 				   unsigned char value)
 {
 	lirc_t pulse;
@@ -326,7 +324,7 @@ static inline void push_full_pulse(struct usb_streamzap *sz,
 					sz->signal_last.tv_usec);
 			tmp -= sz->sum;
 		}
-		dprintk("ls %u", sz->plugin.minor, tmp);
+		dprintk("ls %u", sz->driver.minor, tmp);
 		push(sz, (char *)&tmp);
 
 		sz->idle = 0;
@@ -338,17 +336,17 @@ static inline void push_full_pulse(struct usb_streamzap *sz,
 	sz->sum += pulse;
 	pulse |= PULSE_BIT;
 
-	dprintk("p %u", sz->plugin.minor, pulse&PULSE_MASK);
+	dprintk("p %u", sz->driver.minor, pulse&PULSE_MASK);
 	push(sz, (char *)&pulse);
 }
 
-static inline void push_half_pulse(struct usb_streamzap *sz,
+static void push_half_pulse(struct usb_streamzap *sz,
 				   unsigned char value)
 {
 	push_full_pulse(sz, (value & STREAMZAP_PULSE_MASK)>>4);
 }
 
-static inline void push_full_space(struct usb_streamzap *sz,
+static void push_full_space(struct usb_streamzap *sz,
 				   unsigned char value)
 {
 	lirc_t space;
@@ -356,17 +354,17 @@ static inline void push_full_space(struct usb_streamzap *sz,
 	space = ((lirc_t) value)*STREAMZAP_RESOLUTION;
 	space += STREAMZAP_RESOLUTION/2;
 	sz->sum += space;
-	dprintk("s %u", sz->plugin.minor, space);
+	dprintk("s %u", sz->driver.minor, space);
 	push(sz, (char *)&space);
 }
 
-static inline void push_half_space(struct usb_streamzap *sz,
+static void push_half_space(struct usb_streamzap *sz,
 				   unsigned char value)
 {
 	push_full_space(sz, value & STREAMZAP_SPACE_MASK);
 }
 
-/*
+/**
  * usb_streamzap_irq - IRQ handler
  *
  * This procedure is invoked on reception of data from
@@ -392,18 +390,20 @@ static void usb_streamzap_irq(struct urb *urb)
 	case -ECONNRESET:
 	case -ENOENT:
 	case -ESHUTDOWN:
-		/* this urb is terminated, clean up */
-		/* sz might already be invalid at this point */
+		/*
+		 * this urb is terminated, clean up.
+		 * sz might already be invalid at this point
+		 */
 		dprintk("urb status: %d", -1, urb->status);
 		return;
 	default:
 		break;
 	}
 
-	dprintk("received %d", sz->plugin.minor, urb->actual_length);
+	dprintk("received %d", sz->driver.minor, urb->actual_length);
 	if (!sz->flush) {
 		for (i = 0; i < urb->actual_length; i++) {
-			dprintk("%d: %x", sz->plugin.minor,
+			dprintk("%d: %x", sz->driver.minor,
 				i, (unsigned char) sz->buf_in[i]);
 			switch (sz->decoder_state) {
 			case PulseSpace:
@@ -448,13 +448,18 @@ static void usb_streamzap_irq(struct urb *urb)
 	}
 
 #ifdef KERNEL_2_5
-	/* resubmit only for 2.6 */
 	usb_submit_urb(urb, GFP_ATOMIC);
 #endif
 
 	return;
 }
 
+static struct file_operations streamzap_fops = {
+	.owner		= THIS_MODULE,
+	.ioctl		= streamzap_ioctl,
+};
+
+
 /**
  *	streamzap_probe
  *
@@ -479,20 +484,15 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	struct usb_streamzap *sz = NULL;
 	char buf[63], name[128] = "";
 
-	/***************************************************
-	 * Allocate space for device driver specific data
-	 */
-	sz = kmalloc(sizeof(struct usb_streamzap), GFP_KERNEL);
+	/* Allocate space for device driver specific data */
+	sz = kzalloc(sizeof(struct usb_streamzap), GFP_KERNEL);
 	if (sz == NULL)
 		goto error;
 
-	memset(sz, 0, sizeof(*sz));
 	sz->udev = udev;
 	sz->interface = interface;
 
-	/***************************************************
-	 * Check to ensure endpoint information matches requirements
-	 */
+	/* Check to ensure endpoint information matches requirements */
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 5)
 	iface_host = &interface->altsetting[interface->act_altsetting];
 #else
@@ -505,10 +505,10 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	if (iface_host->bNumEndpoints != 1) {
 #endif
 #ifdef KERNEL_2_5
-		err("%s: Unexpected desc.bNumEndpoints (%d)", __FUNCTION__,
+		err("%s: Unexpected desc.bNumEndpoints (%d)", __func__,
 		    iface_host->desc.bNumEndpoints);
 #else
-		err("%s: Unexpected desc.bNumEndpoints (%d)", __FUNCTION__,
+		err("%s: Unexpected desc.bNumEndpoints (%d)", __func__,
 		    iface_host->bNumEndpoints);
 #endif
 		retval = -ENODEV;
@@ -523,7 +523,7 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	if ((sz->endpoint->bEndpointAddress & USB_ENDPOINT_DIR_MASK)
 	    != USB_DIR_IN) {
 		err("%s: endpoint doesn't match input device 02%02x",
-		    __FUNCTION__, sz->endpoint->bEndpointAddress);
+		    __func__, sz->endpoint->bEndpointAddress);
 		retval = -ENODEV;
 		goto error;
 	}
@@ -531,20 +531,18 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	if ((sz->endpoint->bmAttributes & USB_ENDPOINT_XFERTYPE_MASK)
 	    != USB_ENDPOINT_XFER_INT) {
 		err("%s: endpoint attributes don't match xfer 02%02x",
-		    __FUNCTION__, sz->endpoint->bmAttributes);
+		    __func__, sz->endpoint->bmAttributes);
 		retval = -ENODEV;
 		goto error;
 	}
 
 	if (sz->endpoint->wMaxPacketSize == 0) {
-		err("%s: endpoint message size==0? ", __FUNCTION__);
+		err("%s: endpoint message size==0? ", __func__);
 		retval = -ENODEV;
 		goto error;
 	}
 
-	/***************************************************
-	 * Allocate the USB buffer and IRQ URB
-	 */
+	/* Allocate the USB buffer and IRQ URB */
 
 	sz->buf_in_len = sz->endpoint->wMaxPacketSize;
 #ifdef KERNEL_2_5
@@ -565,9 +563,7 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	if (sz->urb_in == NULL)
 		goto error;
 
-	/***************************************************
-	 * Connect this device to the LIRC sub-system
-	 */
+	/* Connect this device to the LIRC sub-system */
 
 	if (lirc_buffer_init(&sz->lirc_buf, sizeof(lirc_t),
 			     STREAMZAP_BUFFER_SIZE))
@@ -579,26 +575,20 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 		goto error;
 	}
 
-	/***************************************************
-	 * As required memory is allocated now populate the plugin structure
-	 */
-
-	memset(&sz->plugin, 0, sizeof(sz->plugin));
-
-	strcpy(sz->plugin.name, DRIVER_NAME);
-	sz->plugin.minor = -1;
-	sz->plugin.sample_rate = 0;
-	sz->plugin.code_length = sizeof(lirc_t) * 8;
-	sz->plugin.features = LIRC_CAN_REC_MODE2|LIRC_CAN_GET_REC_RESOLUTION;
-	sz->plugin.data = sz;
-	sz->plugin.rbuf = &sz->lirc_buf;
-	sz->plugin.set_use_inc = &streamzap_use_inc;
-	sz->plugin.set_use_dec = &streamzap_use_dec;
-	sz->plugin.ioctl = streamzap_ioctl;
+	strcpy(sz->driver.name, DRIVER_NAME);
+	sz->driver.minor = -1;
+	sz->driver.sample_rate = 0;
+	sz->driver.code_length = sizeof(lirc_t) * 8;
+	sz->driver.features = LIRC_CAN_REC_MODE2 | LIRC_CAN_GET_REC_RESOLUTION;
+	sz->driver.data = sz;
+	sz->driver.rbuf = &sz->lirc_buf;
+	sz->driver.set_use_inc = &streamzap_use_inc;
+	sz->driver.set_use_dec = &streamzap_use_dec;
+	sz->driver.fops = &streamzap_fops;
 #ifdef LIRC_HAVE_SYSFS
-	sz->plugin.dev = &udev->dev;
+	sz->driver.dev = &interface->dev;
 #endif
-	sz->plugin.owner = THIS_MODULE;
+	sz->driver.owner = THIS_MODULE;
 
 	sz->idle = 1;
 	sz->decoder_state = PulseSpace;
@@ -611,32 +601,37 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 	init_timer(&sz->flush_timer);
 	sz->flush_timer.function = flush_timeout;
 	sz->flush_timer.data = (unsigned long) sz;
-	/***************************************************
-	 * Complete final initialisations
-	 */
+	/* Complete final initialisations */
 
 	usb_fill_int_urb(sz->urb_in, udev,
 		usb_rcvintpipe(udev, sz->endpoint->bEndpointAddress),
 		sz->buf_in, sz->buf_in_len, usb_streamzap_irq, sz,
 		sz->endpoint->bInterval);
+#ifdef KERNEL_2_5
+	sz->urb_in->transfer_dma = sz->dma_in;
+	sz->urb_in->transfer_flags |= URB_NO_TRANSFER_DMA_MAP;
+#endif
 
 	if (udev->descriptor.iManufacturer
-	    && usb_string(udev, udev->descriptor.iManufacturer, buf, 63) > 0)
-		strncpy(name, buf, 128);
+	    && usb_string(udev, udev->descriptor.iManufacturer,
+			  buf, sizeof(buf)) > 0)
+		strlcpy(name, buf, sizeof(name));
 
 	if (udev->descriptor.iProduct
-	    && usb_string(udev,  udev->descriptor.iProduct, buf, 63) > 0)
-		snprintf(name, 128, "%s %s", name, buf);
+	    && usb_string(udev,  udev->descriptor.iProduct,
+			  buf, sizeof(buf)) > 0)
+		snprintf(name + strlen(name), sizeof(name) - strlen(name),
+			 " %s", buf);
 
 	printk(KERN_INFO DRIVER_NAME "[%d]: %s on usb%d:%d attached\n",
-	       sz->plugin.minor, name,
+	       sz->driver.minor, name,
 	       udev->bus->busnum, sz->udev->devnum);
 
 #ifdef KERNEL_2_5
 	usb_set_intfdata(interface, sz);
 #endif
 
-	if (lirc_register_plugin(&sz->plugin) < 0) {
+	if (lirc_register_driver(&sz->driver) < 0) {
 		lirc_buffer_free(&sz->delay_buf);
 		lirc_buffer_free(&sz->lirc_buf);
 		goto error;
@@ -650,7 +645,7 @@ static void *streamzap_probe(struct usb_device *udev, unsigned int ifnum,
 
 error:
 
-	/***************************************************
+	/*
 	 * Premise is that a 'goto error' can be invoked from inside the
 	 * probe function and all necessary cleanup actions will be taken
 	 * including freeing any necessary memory blocks
@@ -660,18 +655,14 @@ error:
 		err("Out of memory");
 
 	if (sz) {
-
-		if (sz->urb_in)
-			usb_free_urb(sz->urb_in);
-
-		if (sz->buf_in) {
+		usb_free_urb(sz->urb_in);
 #ifdef KERNEL_2_5
-			usb_buffer_free(udev, sz->buf_in_len,
-					sz->buf_in, sz->dma_in);
+		usb_buffer_free(udev, sz->buf_in_len, sz->buf_in, sz->dma_in);
 #else
+		if (sz->buf_in) {
 			kfree(sz->buf_in);
-#endif
 		}
+#endif
 		kfree(sz);
 	}
 
@@ -687,17 +678,14 @@ static int streamzap_use_inc(void *data)
 	struct usb_streamzap *sz = data;
 
 	if (!sz) {
-		dprintk("%s called with no context", -1, __FUNCTION__);
+		dprintk("%s called with no context", -1, __func__);
 		return -EINVAL;
 	}
-	dprintk("set use inc", sz->plugin.minor);
-
+	dprintk("set use inc", sz->driver.minor);
 	MOD_INC_USE_COUNT;
 
-	while (!lirc_buffer_empty(&sz->lirc_buf))
-		lirc_buffer_remove_1(&sz->lirc_buf);
-	while (!lirc_buffer_empty(&sz->delay_buf))
-		lirc_buffer_remove_1(&sz->delay_buf);
+	lirc_buffer_clear(&sz->lirc_buf);
+	lirc_buffer_clear(&sz->delay_buf);
 
 	sz->flush_timer.expires = jiffies + HZ;
 	sz->flush = 1;
@@ -710,7 +698,7 @@ static int streamzap_use_inc(void *data)
 	if (usb_submit_urb(sz->urb_in)) {
 #endif
 		dprintk("open result = -EIO error submitting urb",
-			sz->plugin.minor);
+			sz->driver.minor);
 		MOD_DEC_USE_COUNT;
 		return -EIO;
 	}
@@ -724,20 +712,20 @@ static void streamzap_use_dec(void *data)
 	struct usb_streamzap *sz = data;
 
 	if (!sz) {
-		dprintk("%s called with no context", -1, __FUNCTION__);
+		dprintk("%s called with no context", -1, __func__);
 		return;
 	}
-	dprintk("set use dec", sz->plugin.minor);
+	dprintk("set use dec", sz->driver.minor);
 
 	if (sz->flush) {
 		sz->flush = 0;
 		del_timer_sync(&sz->flush_timer);
 	}
 
-	stop_timer(sz);
-
 	usb_kill_urb(sz->urb_in);
 
+	stop_timer(sz);
+
 	MOD_DEC_USE_COUNT;
 	sz->in_use--;
 }
@@ -749,9 +737,9 @@ static int streamzap_ioctl(struct inode *node, struct file *filep,
 
 	switch (cmd) {
 	case LIRC_GET_REC_RESOLUTION:
-		result = put_user(STREAMZAP_RESOLUTION, (unsigned long *) arg);
+		result = put_user(STREAMZAP_RESOLUTION, (unsigned int *) arg);
 		if (result)
-			return(result);
+			return result;
 		break;
 	default:
 		return -ENOIOCTLCMD;
@@ -760,14 +748,14 @@ static int streamzap_ioctl(struct inode *node, struct file *filep,
 }
 
 /**
- *	streamzap_disconnect
+ * streamzap_disconnect
  *
- *	Called by the usb core when the device is removed from the system.
+ * Called by the usb core when the device is removed from the system.
  *
- *	This routine guarantees that the driver will not submit any more urbs
- *	by clearing dev->udev.  It is also supposed to terminate any currently
- *	active urbs.  Unfortunately, usb_bulk_msg(), used in streamzap_read(),
- *	does not provide any way to do this.
+ * This routine guarantees that the driver will not submit any more urbs
+ * by clearing dev->udev.  It is also supposed to terminate any currently
+ * active urbs.  Unfortunately, usb_bulk_msg(), used in streamzap_read(),
+ * does not provide any way to do this.
  */
 #ifdef KERNEL_2_5
 static void streamzap_disconnect(struct usb_interface *interface)
@@ -785,21 +773,17 @@ static void streamzap_disconnect(struct usb_device *dev, void *ptr)
 	sz = ptr;
 #endif
 
-	/*
-	 * unregister from the LIRC sub-system
-	 */
+	/* unregister from the LIRC sub-system */
 
-	errnum = lirc_unregister_plugin(sz->plugin.minor);
+	errnum = lirc_unregister_driver(sz->driver.minor);
 	if (errnum != 0)
 		dprintk("error in lirc_unregister: (returned %d)",
-			sz->plugin.minor, errnum);
+			sz->driver.minor, errnum);
 
 	lirc_buffer_free(&sz->delay_buf);
 	lirc_buffer_free(&sz->lirc_buf);
 
-	/*
-	 * unregister from the USB sub-system
-	 */
+	/* unregister from the USB sub-system */
 
 	usb_free_urb(sz->urb_in);
 
@@ -809,7 +793,7 @@ static void streamzap_disconnect(struct usb_device *dev, void *ptr)
 	kfree(sz->buf_in);
 #endif
 
-	minor = sz->plugin.minor;
+	minor = sz->driver.minor;
 	kfree(sz);
 
 	printk(KERN_INFO DRIVER_NAME "[%d]: disconnected\n", minor);
@@ -820,7 +804,7 @@ static int streamzap_suspend(struct usb_interface *intf, pm_message_t message)
 {
 	struct usb_streamzap *sz = usb_get_intfdata(intf);
 
-	printk(DRIVER_NAME "[%d]: suspend\n", sz->plugin.minor);
+	printk(KERN_INFO DRIVER_NAME "[%d]: suspend\n", sz->driver.minor);
 	if (sz->in_use) {
 		if (sz->flush) {
 			sz->flush = 0;
@@ -838,10 +822,8 @@ static int streamzap_resume(struct usb_interface *intf)
 {
 	struct usb_streamzap *sz = usb_get_intfdata(intf);
 
-	while (!lirc_buffer_empty(&sz->lirc_buf))
-		lirc_buffer_remove_1(&sz->lirc_buf);
-	while (!lirc_buffer_empty(&sz->delay_buf))
-		lirc_buffer_remove_1(&sz->delay_buf);
+	lirc_buffer_clear(&sz->lirc_buf);
+	lirc_buffer_clear(&sz->delay_buf);
 
 	if (sz->in_use) {
 		sz->flush_timer.expires = jiffies + HZ;
@@ -855,7 +837,7 @@ static int streamzap_resume(struct usb_interface *intf)
 		if (usb_submit_urb(sz->urb_in)) {
 #endif
 			dprintk("open result = -EIO error submitting urb",
-				sz->plugin.minor);
+				sz->driver.minor);
 			MOD_DEC_USE_COUNT;
 			return -EIO;
 		}
@@ -865,7 +847,6 @@ static int streamzap_resume(struct usb_interface *intf)
 #endif
 
 #ifdef MODULE
-
 /**
  *	usb_streamzap_init
  */
@@ -874,7 +855,6 @@ static int __init usb_streamzap_init(void)
 	int result;
 
 	/* register this driver with the USB subsystem */
-
 	result = usb_register(&streamzap_driver);
 
 	if (result) {
@@ -892,7 +872,6 @@ static int __init usb_streamzap_init(void)
  */
 static void __exit usb_streamzap_exit(void)
 {
-	/* deregister this driver with the USB subsystem */
 	usb_deregister(&streamzap_driver);
 }
 
@@ -904,9 +883,7 @@ MODULE_AUTHOR("Christoph Bartelmus, Greg Wickham, Adrian Dewhurst");
 MODULE_DESCRIPTION(DRIVER_DESC);
 MODULE_LICENSE("GPL");
 
-module_param(debug, bool, 0644);
+module_param(debug, bool, S_IRUGO | S_IWUSR);
 MODULE_PARM_DESC(debug, "Enable debugging messages");
-
 EXPORT_NO_SYMBOLS;
-
 #endif /* MODULE */
